# -*- encoding: utf-8 -*-
from abjad.tools import mathtools
from abjad.tools import sequencetools
from abjad.tools.abctools import AbjadObject
from abjad.tools.topleveltools import iterate
from abjad.tools.topleveltools import mutate
from abjad.tools.pitchtools.PitchArrayCell import PitchArrayCell
from abjad.tools.pitchtools.PitchArrayColumn import PitchArrayColumn
from abjad.tools.pitchtools.PitchArrayRow import PitchArrayRow
from abjad.tools.pitchtools.NamedPitch import NamedPitch


class PitchArray(AbjadObject):
    r'''A two-dimensional array of pitches.
    '''

    ### INITIALIZER ###

    def __init__(self, *args):
        self._rows = []
        self._columns = []
        if len(args) == 1:
            if isinstance(args[0], (tuple, list)):
                self._initialize_by_cell_token_lists(*args)
        elif len(args) == 2:
            if all(isinstance(arg, int) for arg in args):
                self._initialize_by_counts(*args)

    ### SPECIAL METHODS ###

    def __add__(self, arg):
        r'''Concatenates `arg` to pitch array.

        Returns new pitch array.
        '''
        if not isinstance(arg, PitchArray):
            message = 'must be pitch array.'
            raise TypeError(message)
        if not self.depth == arg.depth:
            message = 'array depth must match.'
            raise ValueError(message)
        new_array = PitchArray([])
        for self_row, arg_row in zip(self.rows, arg.rows):
            new_row = self_row + arg_row
            new_array.append_row(new_row)
        return new_array

    def __contains__(self, arg):
        r'''Is true when pitch array contains `arg`. Otherwise false.

        Returns boolean.
        '''
        if isinstance(arg, PitchArrayRow):
            return arg in self.rows
        elif isinstance(arg, PitchArrayColumn):
            return arg in self.columns
        elif isinstance(arg, PitchArrayCell):
            return arg in self.cells
        elif isinstance(arg, NamedPitch):
            for pitch in self.pitches:
                if arg == pitch:
                    return True
            return False
        else:
            message = 'must be row, column, pitch or pitch cell.'
            raise ValueError(message)

    def __copy__(self):
        r'''Copies pitch array.

        Returns new pitch array.
        '''
        return PitchArray(self.cell_tokens_by_row)

    def __eq__(self, arg):
        r'''Is true when `arg` is a pitch aarray with contents equal to that of
        this pitch array. Otherwise false.

        Returns boolean.
        '''
        if isinstance(arg, PitchArray):
            for self_row, arg_row in zip(self.rows, arg.rows):
                if not self_row == arg_row:
                    return False
                return True
        return False

    def __getitem__(self, arg):
        r'''Gets row `arg` from pitch array.

        Returns pitch array row.
        '''
        return self.rows[arg]

    def __getstate__(self):
        r'''Gets object state.
        '''
        return vars(self)

    def __hash__(self):
        r'''Hashes pitch array.

        Required to be explicitely re-defined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(PitchArray, self).__hash__()

    def __iadd__(self, arg):
        r'''Adds `arg` to pitch array in place.

            >>> array_1 = pitchtools.PitchArray([[1, 2, 1], [2, 1, 1]])
            >>> print(array_1)
            [ ] [     ] [ ]
            [     ] [ ] [ ]

        ::

            >>> array_2 = pitchtools.PitchArray([[3, 4], [4, 3]])
            >>> print(array_2)
            [     ] [           ]
            [           ] [     ]

        ::

            >>> array_3 = pitchtools.PitchArray([[1, 1], [1, 1]])
            >>> print(array_3)
            [ ] [ ]
            [ ] [ ]

        ::

            >>> array_1 += array_2
            >>> print(array_1)
            [ ] [     ] [ ] [     ] [         ]
            [     ] [ ] [ ] [         ] [     ]

        ::

            >>> array_1 += array_3
            >>> print(array_1)
            [ ] [     ] [ ] [     ] [         ] [ ] [ ]
            [     ] [ ] [ ] [         ] [     ] [ ] [ ]

        Returns pitch array.
        '''
        if not isinstance(arg, PitchArray):
            message = 'must be pitch array.'
            raise TypeError(message)
        for self_row, arg_row in zip(self.rows, arg.rows):
            self_row += arg_row
        return self

    def __ne__(self, arg):
        r'''Is true when pitch array does not equal `arg`. Otherwise false.

        Returns boolean.
        '''
        return not self == arg

    def __repr__(self):
        r'''Intepreter representation of pitch array.

        Returns string.
        '''
        rows = self.rows
        rows = [repr(row) for row in rows]
        rows = ', '.join(rows)
        return '{}({})'.format(type(self).__name__, rows)

    def __setitem__(self, i, arg):
        r'''Sets pitch array row `i` to `arg`.

        Retunrs none.
        '''
        if isinstance(i, int):
            if not isinstance(arg, PitchArrayRow):
                message = 'can assign only pitch array row to pitch array.'
                raise TypeError(message)
            self._rows[i]._parent_array = None
            arg._parent_array = self
            self._rows[i] = arg
        else:
            message = 'must be integer index.'
            raise ValueError(message)

    def __str__(self):
        r'''String representation of pitch array.

        Returns string.
        '''
        return self._two_by_two_format_string

    ### PRIVATE PROPERTIES ###

    @property
    def _two_by_two_format_string(self):
        return '\n'.join([str(x) for x in self.rows])

    ### PRIVATE METHODS ###

    def _column_format_width_at_index(self, index):
        columns = self.columns
        column = columns[index]
        return column._column_format_width

    def _format_cells(self, cells):
        result = [str(cell) for cell in cells]
        result = ' '.join(result)
        return result

    @staticmethod
    def _get_composite_offset_difference_series_from_leaves_in_expr(expr):
        from abjad.tools import scoretools
        offsets = []
        for leaf in iterate(expr).by_class(scoretools.Leaf):
            start_offset = leaf._get_timespan().start_offset
            if start_offset not in offsets:
                offsets.append(start_offset)
            stop_offset = leaf._get_timespan().stop_offset
            if stop_offset not in offsets:
                offsets.append(stop_offset)
        offsets.sort()
        return list(mathtools.difference_series(offsets))

    def _initialize_by_cell_token_lists(self, cell_token_lists):
        for cell_token_list in cell_token_lists:
            row = PitchArrayRow([])
            for cell_token in cell_token_list:
                cell = self._parse_cell_token(cell_token)
                row.append(cell)
            self.append_row(row)

    def _initialize_by_counts(self, row_count, column_count):
        for i in range(row_count):
            row = PitchArrayRow([])
            for j in range(column_count):
                cell = PitchArrayCell()
                row.append(cell)
            self.append_row(row)

    def _parse_cell_token(self, cell_token):
        return PitchArrayCell(cell_token)

    ### PUBLIC PROPERTIES ###

    @property
    def cell_tokens_by_row(self):
        r'''Cells items of pitch array by row.

        Returns tuple.
        '''
        return tuple([row.cell_tokens for row in self.rows])

    @property
    def cell_widths_by_row(self):
        r'''Cell widths of pitch array by row.

        Returns tuple.
        '''
        return tuple([row.cell_widths for row in self.rows])

    @property
    def cells(self):
        r'''Cells of pitch array.

        Returns set.
        '''
        cells = set([])
        for row in self.rows:
            cells.update(row.cells)
        return cells

    @property
    def columns(self):
        r'''Columns of pitch array.

        Returns tuple.
        '''
        columns = []
        rows = self.rows
        for i, cells in enumerate(
            sequencetools.zip_sequences(self.rows, truncate=False)):
            column = PitchArrayColumn(cells)
            column._parent_array = self
            column._column_index = i
            columns.append(column)
        return tuple(columns)

    @property
    def depth(self):
        r'''Depth of pitch array.

        Defined equal to number of pitch array rows in pitch array.

        Returns nonnegative integer.
        '''
        return len(self.rows)

    @property
    def dimensions(self):
        r'''Dimensions of pitch array.

        Returns pair.
        '''
        return self.depth, self.width

    @property
    def has_voice_crossing(self):
        r'''Is true when pitch array has voice crossing. Otherwise false.

        Returns boolean.
        '''
        for column in self.columns:
            if column.has_voice_crossing:
                return True
        return False

    @property
    def is_rectangular(self):
        r'''Is true when no rows in pitch array are defective. Otherwise false.

        Returns boolean.
        '''
        return all(not row.is_defective for row in self.rows)

    @property
    def pitches(self):
        r'''Pitches in pitch array.

        Returns tuple.
        '''
        return sequencetools.flatten_sequence(self.pitches_by_row)

    @property
    def pitches_by_row(self):
        r'''Pitches in pitch array by row.

        Returns tuple.
        '''
        pitches = []
        for row in self.rows:
            pitches.append(row.pitches)
        return tuple(pitches)

    @property
    def rows(self):
        r'''Rows in pitch array.

        Returns tuple.
        '''
        return tuple(self._rows)

    @property
    def size(self):
        r'''Size of pitch array.

        Defined equal to the product of depth and width.

        Returns nonnegative integer.
        '''
        return self.depth * self.width

    @property
    def voice_crossing_count(self):
        r'''Voice crossing count.

        Returns nonnegative integer.
        '''
        count = 0
        for column in self.columns:
            if column.has_voice_crossing:
                count += 1
        return count

    @property
    def weight(self):
        r'''Weight of pitch array.

        Defined equal to the sum of the weight of the rows in pitch array.

        Returns nonnegative integer.
        '''
        return sum([row.weight for row in self.rows])

    @property
    def width(self):
        r'''Width of pitch array.

        Defined equal to the width of the widest row in pitch array.

        Returns nonnegative integer.
        '''
        try:
            return max([row.width for row in self.rows])
        except ValueError:
            return 0

    ### PUBLIC METHODS ###

    def append_column(self, column):
        r'''Append `column` to pitch array.

        Returns none.
        '''
        if not isinstance(column, PitchArrayColumn):
            message = 'must be column.'
            raise TypeError(message)
        column._parent_array = self
        column_depth = column.depth
        if self.depth < column_depth:
            self.pad_to_depth(column_depth)
        self.pad_to_width(self.width)
        for row, cell in zip(self.rows, column):
            row.append(cell)

    def append_row(self, row):
        r'''Appends `row` to pitch array.

        Returns none.
        '''
        if not isinstance(row, PitchArrayRow):
            message = 'must be row.'
            raise TypeError(message)
        row._parent_array = self
        self._rows.append(row)

    def apply_pitches_by_row(self, pitch_lists):
        r'''Applies `pitch_lists` to pitch array by row.

        Returns none.
        '''
        for row, pitch_list in zip(self.rows, pitch_lists):
            row.apply_pitches(pitch_list)

    def copy_subarray(self, upper_left_pair, lower_right_pair):
        r'''Copies subarray of pitch array.

        Returns new pitch array.
        '''
        if not isinstance(upper_left_pair, tuple):
            raise TypeError
        if not isinstance(lower_right_pair, tuple):
            raise TypeError
        start_i, start_j = upper_left_pair
        stop_i, stop_j = lower_right_pair
        if not start_i <= stop_i:
            message = 'start row must not be greater than stop row.'
            raise ValueError(message)
        if not start_j <= stop_j:
            message = 'start column must not be greater than stop column.'
            raise ValueError(message)
        new_array = PitchArray([])
        rows = self.rows
        row_indices = range(start_i, stop_i)
        for row_index in row_indices:
            new_row = rows[row_index].copy_subrow(start_j, stop_j)
            new_array.append_row(new_row)
        return new_array

    @staticmethod
    def from_score(score, populate=True):
        r'''Makes pitch array from `score`.

        ..  container:: example

            **Example 1.** Makes empty pitch array from score:

            ::

                >>> score = Score([])
                >>> score.append(Staff("c'8 d'8 e'8 f'8"))
                >>> score.append(Staff("c'4 d'4"))
                >>> score.append(
                ...     Staff(
                ...     scoretools.FixedDurationTuplet(
                ...     Duration(2, 8), "c'8 d'8 e'8") * 2))

            ..  doctest::

                >>> print(format(score))
                \new Score <<
                    \new Staff {
                        c'8
                        d'8
                        e'8
                        f'8
                    }
                    \new Staff {
                        c'4
                        d'4
                    }
                    \new Staff {
                        \times 2/3 {
                            c'8
                            d'8
                            e'8
                        }
                        \times 2/3 {
                            c'8
                            d'8
                            e'8
                        }
                    }
                >>

            ::

                >>> show(score) # doctest: +SKIP

            ::

                >>> array = pitchtools.PitchArray.from_score(
                ...     score, populate=False)

            ::

                >>> print(array)
                [     ] [     ] [     ] [     ]
                [                 ] [                 ]
                [ ] [     ] [ ] [ ] [     ] [ ]

        ..  container:: example

            **Example 2.** Makes populated pitch array from `score`:

            ::

                >>> score = Score([])
                >>> score.append(Staff("c'8 d'8 e'8 f'8"))
                >>> score.append(Staff("c'4 d'4"))
                >>> score.append(
                ...     Staff(
                ...     scoretools.FixedDurationTuplet(
                ...     Duration(2, 8), "c'8 d'8 e'8") * 2))

            ..  doctest::

                >>> print(format(score))
                \new Score <<
                    \new Staff {
                        c'8
                        d'8
                        e'8
                        f'8
                    }
                    \new Staff {
                        c'4
                        d'4
                    }
                    \new Staff {
                        \times 2/3 {
                            c'8
                            d'8
                            e'8
                        }
                        \times 2/3 {
                            c'8
                            d'8
                            e'8
                        }
                    }
                >>

            ::

                >>> show(score) # doctest: +SKIP

            ::

                >>> array = pitchtools.PitchArray.from_score(
                ...     score, populate=True)

            ::

                >>> print(array)
                [c'     ] [d'     ] [e'     ] [f'     ]
                [c'                   ] [d'                   ]
                [c'] [d'     ] [e'] [c'] [d'     ] [e']

        Returns pitch array.
        '''
        from abjad.tools import pitchtools
        from abjad.tools import scoretools
        time_intervals = \
            PitchArray._get_composite_offset_difference_series_from_leaves_in_expr(
            score)
        array_width = len(time_intervals)
        array_depth = len(score)
        pitch_array = PitchArray(array_depth, array_width)
        items = scoretools.make_multiplied_quarter_notes(
            [0], time_intervals)
        for leaf_iterable, pitch_array_row in \
            zip(score, pitch_array.rows):
            durations = []
            for leaf in iterate(leaf_iterable).by_class(scoretools.Leaf):
                durations.append(leaf._get_duration())
            parts = mutate(items).split(
                durations,
                cyclic=False,
                fracture_spanners=False,
                )
            part_lengths = [len(part) for part in parts]
            cells = pitch_array_row.cells
            grouped_cells = sequencetools.partition_sequence_by_counts(
                cells,
                part_lengths,
                cyclic=False,
                overhang=False,
                )
            for group in grouped_cells:
                pitch_array_row.merge(group)
            leaves = iterate(leaf_iterable).by_class(scoretools.Leaf)
            if populate:
                for cell, leaf in zip(pitch_array_row.cells, leaves):
                    cell.pitches.extend(
                        pitchtools.list_named_pitches_in_expr(leaf))
        return pitch_array

    def has_spanning_cell_over_index(self, index):
        r'''Is true when pitch array has one or more spanning cells over `index`.
        Otherwise false.

        Returns boolean.
        '''
        rows = self.rows
        return any(row.has_spanning_cell_over_index(index) for row in rows)

    def list_nonspanning_subarrays(self):
        r'''Lists nonspanning subarrays of pitch array.

        ::

            >>> array = pitchtools.PitchArray([
            ...     [2, 2, 3, 1],
            ...     [1, 2, 1, 1, 2, 1],
            ...     [1, 1, 1, 1, 1, 1, 1, 1]])
            >>> print(array)
            [     ] [     ] [         ] [ ]
            [ ] [     ] [ ] [ ] [     ] [ ]
            [ ] [ ] [ ] [ ] [ ] [ ] [ ] [ ]

        ::

            >>> subarrays = array.list_nonspanning_subarrays()
            >>> len(subarrays)
            3

        ::

            >>> print(subarrays[0])
            [     ] [     ]
            [ ] [     ] [ ]
            [ ] [ ] [ ] [ ]

        ::

            >>> print(subarrays[1])
            [         ]
            [ ] [     ]
            [ ] [ ] [ ]

        ::

            >>> print(subarrays[2])
            [ ]
            [ ]
            [ ]

        Returns list.
        '''
        unspanned_indices = []
        for i in range(self.width + 1):
            if not self.has_spanning_cell_over_index(i):
                unspanned_indices.append(i)
        array_depth = self.depth
        subarrays = []
        for start_column, stop_column in \
            sequencetools.iterate_sequence_nwise(unspanned_indices):
            upper_left_pair = (0, start_column)
            lower_right_pair = (array_depth, stop_column)
            subarray = self.copy_subarray(upper_left_pair, lower_right_pair)
            subarrays.append(subarray)
        return subarrays

    def pad_to_depth(self, depth):
        r'''Pads pitch array to `depth`.

        Returns none.
        '''
        self_depth = self.depth
        if depth < self_depth:
            message = 'pad depth must be not less than array depth.'
            raise ValueError(message)
        self_width = self.width
        missing_rows = depth - self_depth
        for i in range(missing_rows):
            row = PitchArrayRow([])
            row.pad_to_width(self_width)
            self.append_row(row)

    def pad_to_width(self, width):
        r'''Pads pitch array to `width`.

        Returns none.
        '''
        self_width = self.width
        if width < self_width:
            message = 'pad width must not be less than array width.'
            raise ValueError(message)
        for row in self.rows:
            row.pad_to_width(width)

    def pop_column(self, column_index):
        r'''Pops column `column_index` from pitch array.

        Returns pitch array column.
        '''
        column = self.columns[column_index]
        column._parent_array = None
        for cell in column.cells:
            cell.withdraw()
        return column

    def pop_row(self, row_index=-1):
        r'''Pops row `row_index` from pitch array.

        Returns pitch array row.
        '''
        row = self._rows.pop(row_index)
        row._parent_array = None
        return row

    def remove_row(self, row):
        r'''Removes `row` from pitch array.

        Returns none.
        '''
        if row not in self.rows:
            message = 'row not in array.'
            raise ValueError(message)
        self._rows.remove(row)
        row._parent_array = None

    def to_measures(self, cell_duration_denominator=8):
        r'''Changes pitch array  to measures with time signatures
        with numerators equal to row width and denominators
        equal to `cell_duration_denominator` for each row
        in pitch array.

        ::

            >>> array = pitchtools.PitchArray([
            ...     [1, (2, 1), ([-2, -1.5], 2)],
            ...     [(7, 2), (6, 1), 1]])

        ::

            >>> print(array)
            [  ] [d'] [bf bqf    ]
            [g'     ] [fs'   ] [ ]

        ::

            >>> measures = array.to_measures()

        ::

            >>> for measure in measures:
            ...     f(measure)
            ...
            {
                \time 4/8
                r8
                d'8
                <bf bqf>4
            }
            {
                \time 4/8
                g'4
                fs'8
                r8
            }

        Returns list of measures.
        '''
        measures = []
        for row in self.rows:
            measure = row.to_measure(cell_duration_denominator)
            measures.append(measure)
        return measures



# -*- encoding: utf-8 -*-
from abjad import *


def test_Sequence_rotate_01():
    r'''Rotates sequence by distance less than or equal to sequence length.
    '''

    sequence = sequencetools.Sequence(*range(10))
    assert sequence.rotate(-3) == \
        sequencetools.Sequence(3, 4, 5, 6, 7, 8, 9, 0, 1, 2)
    assert sequence.rotate(4) == \
        sequencetools.Sequence(6, 7, 8, 9, 0, 1, 2, 3, 4, 5)
    assert sequence.rotate(0) == \
        sequencetools.Sequence(0, 1, 2, 3, 4, 5, 6, 7, 8, 9)


def test_Sequence_rotate_02():
    r'''Rotates sequence by distance greatern than sequence length.
    '''

    sequence = sequencetools.Sequence(*range(10))
    assert sequence.rotate(-23) == \
        sequencetools.Sequence(3, 4, 5, 6, 7, 8, 9, 0, 1, 2)
    assert sequence.rotate(24) == \
        sequencetools.Sequence(6, 7, 8, 9, 0, 1, 2, 3, 4, 5)


def test_Sequence_rotate_03():
    r'''Returns sequence type.
    '''

    sequence = sequencetools.Sequence(*range(10))
    new = sequence.rotate(-1)
    assert isinstance(new, type(sequence))
    new = sequence.rotate(-1)
    assert isinstance(new, type(sequence))


# -*- encoding: utf-8 -*-
from abjad.tools import indicatortools
from abjad.tools import lilypondfiletools
from abjad.tools import scoretools
from abjad.tools import selectiontools
from abjad.tools import sequencetools
from abjad.tools.topleveltools import mutate


def make_lilypond_file(music, divisions, implicit_scaling=False):
    r'''Makes LilyPond file.

    ..  container::

        ::

            >>> maker = rhythmmakertools.EvenRunRhythmMaker(1)
            >>> divisions = [(3, 4), (4, 8), (1, 4)]
            >>> music = maker(divisions)
            >>> lilypond_file = rhythmmakertools.make_lilypond_file(
            ...     music,
            ...     divisions,
            ...     )
            >>> show(lilypond_file) # doctest: +SKIP

    Used in rhythm-maker docs.

    Returns LilyPond file.
    '''

    assert isinstance(music, list), repr(music)
    prototype = (selectiontools.Selection, scoretools.Tuplet)
    assert all(isinstance(x, prototype) for x in music), repr(music)
    assert isinstance(divisions, (tuple, list)), repr(divisions)

    score = scoretools.Score()
    lilypond_file = \
        lilypondfiletools.make_floating_time_signature_lilypond_file(score)

    context = scoretools.Context(context_name='TimeSignatureContext')
    measures = scoretools.make_spacer_skip_measures(
        divisions,
        implicit_scaling=implicit_scaling,
        )
    context.extend(measures)
    score.append(context)

    measures = scoretools.make_spacer_skip_measures(
        divisions,
        implicit_scaling=implicit_scaling,
        )
    staff = scoretools.Staff(measures)
    staff.context_name = 'RhythmicStaff'
    music = sequencetools.flatten_sequence(music)

    measures = mutate(staff).replace_measure_contents(music)
    score.append(staff)

    return lilypond_file


from abjad.tools.pitchtools.Vector import Vector


class PitchClassVector(Vector):
    '''A pitch-class vector.
    '''

    ### PRIVATE PROPERTIES ###

    @property
    def _named_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.NamedPitchClass

    @property
    def _numbered_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.NumberedPitchClass

    @property
    def _parent_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.PitchClass

    ### PUBLIC METHODS ###

    @classmethod
    def from_selection(
        cls,
        selection,
        item_class=None,
        ):
        r'''Makes pitch-class vector from `selection`.

        Returns pitch-class vector.
        '''
        from abjad.tools import pitchtools
        pitch_segment = pitchtools.PitchSegment.from_selection(selection)
        return cls(
            pitch_segment,
            item_class=item_class,
            )


# -*- encoding: utf-8 -*-
import pytest
from abjad import *


def test_mathtools_least_power_of_two_greater_equal_01():
    r'''Returns least integer power of two
    greater than or equal to n.
    '''

    assert mathtools.least_power_of_two_greater_equal(1) == 1
    assert mathtools.least_power_of_two_greater_equal(2) == 2
    assert mathtools.least_power_of_two_greater_equal(3) == 4
    assert mathtools.least_power_of_two_greater_equal(4) == 4
    assert mathtools.least_power_of_two_greater_equal(5) == 8
    assert mathtools.least_power_of_two_greater_equal(6) == 8
    assert mathtools.least_power_of_two_greater_equal(7) == 8
    assert mathtools.least_power_of_two_greater_equal(8) == 8
    assert mathtools.least_power_of_two_greater_equal(9) == 16
    assert mathtools.least_power_of_two_greater_equal(10) == 16
    assert mathtools.least_power_of_two_greater_equal(11) == 16
    assert mathtools.least_power_of_two_greater_equal(12) == 16



def test_mathtools_least_power_of_two_greater_equal_02():
    r'''Raise TypeError on nonnumeric n.
    Raise ValueError on nonpositive n.
    '''

    statement = "mathtools.least_power_of_two_greater_equal('foo')"
    assert pytest.raises(TypeError, statement)
    statement = 'mathtools.least_power_of_two_greater_equal(0)'
    assert pytest.raises(ValueError, statement)
    statement = 'mathtools.least_power_of_two_greater_equal(-1)'
    assert pytest.raises(ValueError, statement)


def test_mathtools_least_power_of_two_greater_equal_03():
    r'''Optional offset keyword allows for the next to greatest
    integer power of two, etc.
    '''

    assert mathtools.least_power_of_two_greater_equal(1, 1) == 2
    assert mathtools.least_power_of_two_greater_equal(2, 1) == 4
    assert mathtools.least_power_of_two_greater_equal(3, 1) == 8
    assert mathtools.least_power_of_two_greater_equal(4, 1) == 8
    assert mathtools.least_power_of_two_greater_equal(5, 1) == 16
    assert mathtools.least_power_of_two_greater_equal(6, 1) == 16
    assert mathtools.least_power_of_two_greater_equal(7, 1) == 16
    assert mathtools.least_power_of_two_greater_equal(8, 1) == 16
    assert mathtools.least_power_of_two_greater_equal(9, 1) == 32
    assert mathtools.least_power_of_two_greater_equal(10, 1) == 32
    assert mathtools.least_power_of_two_greater_equal(11, 1) == 32
    assert mathtools.least_power_of_two_greater_equal(12, 1) == 32


# -*- encoding: utf-8 -*-
from abjad.tools.abctools import AbjadObject


class BowPressure(AbjadObject):
    r'''Bow pressure indicator.

    ::

        >>> indicator = indicatortools.BowPressure('overpressure')
        >>> print(format(indicator))
        indicatortools.BowPressure(
            pressure='overpressure',
            )

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_pressure',
        )

    ### INITIALIZER ###

    def __init__(self,
        pressure=None,
        ):
        self._pressure = pressure

    ### PUBLIC PROPERTIES ###

    @property
    def pressure(self):
        r'''Gets pressure.

        ::

            >>> indicator = indicatortools.BowPressure('underpressure')
            >>> indicator.pressure
            'underpressure'

        '''
        return self._pressure



# -*- encoding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchClassTree___illustrate___01():


    tree = pitchtools.PitchClassTree(
        items=[['c', 'e', 'g', 'af'], ['a', ['d', 'ef'], 'b']],
        item_class=pitchtools.NamedPitchClass,
        )
    lilypond_file = tree.__illustrate__()
    score = lilypond_file.score_block.items[0]

    assert systemtools.TestManager.compare(
        score,
        r'''
        \new Score \with {
            \override BarLine #'stencil = ##f
            \override Flag #'stencil = ##f
            \override Stem #'stencil = ##f
            \override TextScript #'staff-padding = #3
            \override TimeSignature #'stencil = ##f
        } <<
            \new Staff {
                \new Voice \with {
                    \consists Horizontal_bracket_engraver
                } {
                    c'8 \startGroup
                        ^ \markup {
                            \bold
                                {
                                    0
                                }
                            }
                    e'8
                    g'8
                    af'8 \stopGroup
                    a'8 \startGroup
                        ^ \markup {
                            \bold
                                {
                                    1
                                }
                            }
                    d'8 \startGroup
                    ef'8 \stopGroup
                    b'8 \stopGroup
                    \bar "|."
                }
            }
        >>
        '''
        ), format(score)


# -*- encoding: utf-8 -*-


def negate_elements(sequence, absolute=False, indices=None, period=None):
    '''Negates `sequence` elements.

    ..  container:: example

        Negates all elements:

        ::

            >>> sequence = [1, 2, 3, 4, 5, -6, -7, -8, -9, -10]
            >>> sequencetools.negate_elements(sequence)
            [-1, -2, -3, -4, -5, 6, 7, 8, 9, 10]

    ..  container:: example

        Negates elements at indices 0, 1 and 2:

        ::

            >>> sequence = [1, 2, 3, 4, 5, -6, -7, -8, -9, -10]
            >>> sequencetools.negate_elements(sequence, indices=[0, 1, 2])
            [-1, -2, -3, 4, 5, -6, -7, -8, -9, -10]

    ..  container:: example

        Negates elements at indices congruent to 0, 1 or 2 mod 5:

        ::


            >>> sequence = [1, 2, 3, 4, 5, -6, -7, -8, -9, -10]
            >>> sequencetools.negate_elements(
            ...     sequence,
            ...     indices=[0, 1, 2],
            ...     period=5,
            ...     )
            [-1, -2, -3, 4, 5, 6, 7, 8, -9, -10]

    ..  container:: example

        Negates the absolute value of all elements:

        ::

            >>> sequence = [1, 2, 3, 4, 5, -6, -7, -8, -9, -10]
            >>> sequencetools.negate_elements(sequence, absolute=True)
            [-1, -2, -3, -4, -5, -6, -7, -8, -9, -10]

    ..  container:: example

        Negates the absolute value elements at indices 0, 1 and 2:

        ::

            >>> sequence = [1, 2, 3, 4, 5, -6, -7, -8, -9, -10]
            >>> sequencetools.negate_elements(
            ...     sequence,
            ...     absolute=True,
            ...     indices=[0, 1, 2],
            ...     )
            [-1, -2, -3, 4, 5, -6, -7, -8, -9, -10]

    ..  container:: example

        Negates the absolute value elements at indices congruent to 0, 1 or 2
        mod 5:

        ::

            >>> sequence = [1, 2, 3, 4, 5, -6, -7, -8, -9, -10]
            >>> sequencetools.negate_elements(
            ...     sequence,
            ...     absolute=True,
            ...     indices=[0, 1, 2],
            ...     period=5,
            ...     )
            [-1, -2, -3, 4, 5, -6, -7, -8, -9, -10]

    Returns newly constructed list.
    '''

    indices = indices or range(len(sequence))

    if not isinstance(sequence, list):
        raise TypeError

    period = period or len(sequence)

    result = []

    for i, element in enumerate(sequence):
        if (i in indices) or (period and i % period in indices):
            if absolute:
                result.append(-abs(element))
            else:
                result.append(-element)
        else:
            result.append(element)

    return result


# -*- encoding: utf-8 -*-
import fnmatch
import os
import re
from abjad.tools.developerscripttools.DirectoryScript import DirectoryScript


class ReplaceInFilesScript(DirectoryScript):
    r'''Replaces text in files recursively.

    ..  shell::

        ajv replace text --help

    Multiple patterns for excluding files or folders can be specified by
    restating the `--without-files` or `--without-dirs` commands:

    ..  code-block:: bash

        abjad$ ajv replace text . foo bar -F *.txt -F *.rst -F *.htm

    '''

    ### PUBLIC PROPERTIES ###

    @property
    def alias(self):
        r'''Alias of script.

        Returns ``'replace'``.
        '''
        return 'replace'

    @property
    def long_description(self):
        r'''Long description of script.

        Returns string or none.
        '''
        return None

    @property
    def scripting_group(self):
        r'''Scripting group of script.

        Returns none.
        '''
        return None

    @property
    def short_description(self):
        r'''Short description.

        Returns string.
        '''
        return 'Replace text.'

    @property
    def skipped_directories(self):
        r'''Skipped directories.

        Returns list.
        '''
        return [
            '.svn',
            '.git',
            'build'
        ]

    @property
    def skipped_files(self):
        r'''Skipped files.

        Returns list.
        '''
        return [
            __file__,
            self.program_name,
            '*.doctree',
            '*.gif',
            '*.jpg',
            '*.jpeg',
            '*.ly',
            '*.nb',
            '*.pdf',
            '*.pickle',
            '*.pkl',
            '*.png',
            '*.ps',
            '*.pyc',
            '*.rtf',
            '*.tex',
            '*.txt',
        ]

    @property
    def version(self):
        r'''Version of script.

        Returns float.
        '''
        return 1.0

    ### PRIVATE METHODS ###

    def _get_naive_search_callable(self, args):
        class NaiveSearch(object):
            def __init__(self, pattern):
                self.pattern = pattern
            def __call__(self, line, pos):
                index = line.find(self.pattern, pos)
                if 0 <= index:
                    return index, len(self.pattern)
                return -1, 0
        return NaiveSearch(args.old)

    def _get_regex_search_callable(self, args):
        class RegexSearch(object):
            def __init__(self, pattern, escaped=False, whole_words_only=False):
                try:
                    if escaped:
                        pattern = re.escape(pattern)
                    if whole_words_only:
                        pattern += r'\b'
                    self.pattern = re.compile(pattern)
                    self.whole_words_only = whole_words_only
                except:
                    message = "can't compile {!r} as a regex pattern."
                    message = message.format(pattern)
                    raise ValueError(message)

            def __call__(self, line, pos):
                start, length = self._search(line, pos)
                if self.whole_words_only and 0 < start:
                    while start != -1 and (
                        line[start-1].isalnum() or line[start-1] == '_'):
                        start, length = self._search(line, start + length)
                return start, length

            def _search(self, line, pos):
                match = self.pattern.search(line, pos)
                if match is None:
                    return -1, 0
                return match.start(), match.end() - match.start()

        return RegexSearch(
            args.old,
            escaped=not args.regex,
            whole_words_only=args.whole_words_only,
            )

    def _process_file(self, args, path):

        changed_items = 0
        changed_lines = 0

        with open(path, 'r') as f:
            lines = f.read().split('\n')

        results = []
        for i, line in enumerate(lines):
            line, changes = self._process_line(
                line, i, path, args.old, args.new, args.force, args.verbose)
            results.append(line)
            if changes:
                changed_items += changes
                changed_lines += 1

        if results != lines:
            with open(path, 'w') as f:
                f.write('\n'.join(results))

        return changed_lines, changed_items

    def _process_line(
        self,
        line,
        line_number,
        file_name,
        search,
        replacement,
        force,
        verbose,
        ):
        index, changes = 0, 0
        index, length = search(line, index)

        while 0 <= index:

            should_replace = False

            replaced_line = line[:index] + replacement + line[index+length:]
            carats = (' ' * index) + ('^' * length)

            if force:
                should_replace = True
                if verbose:
                    print('')
                    print('{}: {}'.format(file_name, line_number))
                    print('-{}'.format(line))
                    print('+{}'.format(replaced_line))

            else:
                print('')
                print('{}: {}'.format(file_name, line_number))
                print('')
                print('{}'.format(line))
                print('{}'.format(carats))
                print('')
                result = raw_input('Replace? [Y/n] > ').lower()
                while result not in ('', 'y', 'yes', 'n', 'no'):
                    result = raw_input('Replace? [Y/n] > ').lower()
                if result in ('', 'y', 'yes'):
                    should_replace = True

            if should_replace:
                index += length - (length - len(replacement))
                line = replaced_line
                changes += 1
            else:
                index += length

            index, length = search(line, index)

        return line, changes

    ### PUBLIC METHODS ###

    def process_args(self, args):
        r'''Processes `args`.

        Returns none.
        '''
        print('Replacing {!r} with {!r} ...'.format(args.old, args.new))
        skipped_dirs_patterns = self.skipped_directories + args.without_dirs
        skipped_files_patterns = self.skipped_files + args.without_files
        if args.regex or (not args.regex and args.whole_words_only):
            args.old = self._get_regex_search_callable(args)
            index, length = args.old('', 0)
            if 0 <= index:
                message = 'regex pattern {!r} matches the empty string.'
                message = message.format(args.old.pattern.pattern)
                raise ValueError(message)
        else:
            args.old = self._get_naive_search_callable(args)
        changed_file_count = 0
        changed_line_count = 0
        changed_item_count = 0
        for root, dirs, files in os.walk(args.path):
            dirs_to_remove = []
            for dir in dirs:
                for pattern in skipped_dirs_patterns:
                    if fnmatch.fnmatch(dir, pattern):
                        dirs_to_remove.append(dir)
                        break
            for dir in dirs_to_remove:
                dirs.remove(dir)
            for file in sorted(files):
                valid = True
                for pattern in skipped_files_patterns:
                    if fnmatch.fnmatch(file, pattern):
                        valid = False
                        break
                if not valid:
                    continue
                changed_lines, changed_items = self._process_file(
                    args, os.path.join(root, file))
                if changed_lines:
                    changed_file_count += 1
                    changed_line_count += changed_lines
                    changed_item_count += changed_items
        print('')
        print('\tReplaced {} instances over {} lines in {} files.'.format(
            changed_item_count, changed_line_count, changed_file_count))

    def setup_argument_parser(self, parser):
        r'''Sets up argument `parser`.

        Returns none.
        '''
        parser.add_argument('path',
            default=os.getcwd(),
            help='directory tree to be recursed over',
            nargs='?',
            type=self._validate_path,
            )
        parser.add_argument('old',
            help='old text',
            )
        parser.add_argument('new',
            help='new text',
            )
        parser.add_argument('--verbose',
            action='store_true',
            help='print replacement info even when --force flag is set.',
            )
        parser.add_argument('-Y', '--force',
            action='store_true',
            help='force "yes" to every replacement'
            )
        parser.add_argument('-R', '--regex',
            action='store_true',
            help='treat "old" as a regular expression',
            )
        parser.add_argument('-W', '--whole-words-only',
            action='store_true',
            help='''match only whole words, similar to grep's "-w" flag''',
            )
        parser.add_argument('-F', '--without-files',
            action='append',
            default=[],
            help='Exclude files matching pattern(s)',
            metavar='PATTERN',
            )
        parser.add_argument('-D', '--without-dirs',
            action='append',
            default=[],
            help='Exclude folders matching pattern(s)',
            metavar='PATTERN',
            )


# -*- encoding: utf-8 -*-


def cumulative_sums_pairwise(sequence):
    r'''Lists pairwise cumulative sums of `sequence` from ``0``.

    ::

        >>> mathtools.cumulative_sums_pairwise([1, 2, 3, 4, 5, 6])
        [(0, 1), (1, 3), (3, 6), (6, 10), (10, 15), (15, 21)]

    Returns list of pairs.
    '''
    from abjad.tools import mathtools
    from abjad.tools import sequencetools

    sums = mathtools.cumulative_sums(sequence)
    return list(sequencetools.iterate_sequence_nwise(sums))


# -*- encoding: utf-8 -*-
import fractions
from abjad.tools import mathtools


def partition_sequence_by_ratio_of_weights(sequence, weights):
    '''Partitions `sequence` by ratio of `weights`.

    ::

        >>> sequencetools.partition_sequence_by_ratio_of_weights(
        ...     [1] * 10, [1, 1, 1])
        [[1, 1, 1], [1, 1, 1, 1], [1, 1, 1]]

    ::

        >>> sequencetools.partition_sequence_by_ratio_of_weights(
        ...     [1] * 10, [1, 1, 1, 1])
        [[1, 1, 1], [1, 1], [1, 1, 1], [1, 1]]

    ::

        >>> sequencetools.partition_sequence_by_ratio_of_weights(
        ...     [1] * 10, [2, 2, 3])
        [[1, 1, 1], [1, 1, 1], [1, 1, 1, 1]]

    ::

        >>> sequencetools.partition_sequence_by_ratio_of_weights(
        ...     [1] * 10, [3, 2, 2])
        [[1, 1, 1, 1], [1, 1, 1], [1, 1, 1]]

    ::

        >>> sequencetools.partition_sequence_by_ratio_of_weights(
        ...     [1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2], [1, 1])
        [[1, 1, 1, 1, 1, 1, 2, 2], [2, 2, 2, 2]]

    ::

        >>> sequencetools.partition_sequence_by_ratio_of_weights(
        ...     [1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2], [1, 1, 1])
        [[1, 1, 1, 1, 1, 1], [2, 2, 2], [2, 2, 2]]


    Weights of parts of returned list equal `weights_ratio` proportions
    with some rounding magic.

    Returns list of lists.
    '''
    from abjad.tools import sequencetools

    list_weight = mathtools.weight(sequence)
    weights_parts = mathtools.partition_integer_by_ratio(list_weight, weights)
    cumulative_weights = mathtools.cumulative_sums(weights_parts, start=None)

    result = []
    sublist = []
    result.append(sublist)
    current_cumulative_weight = cumulative_weights.pop(0)
    for n in sequence:
        if not isinstance(n, (int, float, fractions.Fraction)):
            message = 'must be number: {!r}.'
            message = message.format(n)
            raise TypeError(message)
        sublist.append(n)
        while current_cumulative_weight <= \
            mathtools.weight(sequencetools.flatten_sequence(result)):
            try:
                current_cumulative_weight = cumulative_weights.pop(0)
                sublist = []
                result.append(sublist)
            except IndexError:
                break

    return result


# -*- encoding: utf-8 -*-
import functools
from abjad.tools import durationtools
from abjad.tools import markuptools
from abjad.tools.abctools import AbjadObject


@functools.total_ordering
class BowContactPoint(AbjadObject):
    r'''Bow contact point indicator.

    Contact points are measured from frog to tip as a fraction between 0 and 1.

    ::

        >>> indicator = indicatortools.BowContactPoint((1, 2))
        >>> print(format(indicator))
        indicatortools.BowContactPoint(
            contact_point=durationtools.Multiplier(1, 2),
            )

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_contact_point',
        )

    ### INITIALIZER ###

    def __init__(self,
        contact_point=None,
        ):
        if contact_point is not None:
            contact_point = durationtools.Multiplier(contact_point)
            assert 0 <= contact_point <= 1
        self._contact_point = contact_point

    ### SPECIAL METHODS ###

    def __eq__(self, expr):
        r'''Is true if `expr` is a bow contact point with the same contact
        point as this bow contact point.

        Returns boolean.
        '''
        if isinstance(expr, type(self)):
            return self.contact_point == expr.contact_point
        raise TypeError('unorderable types')

    def __hash__(self):
        r'''Hashes bow contact point.
        '''
        from abjad.tools import systemtools
        return hash(systemtools.StorageFormatManager.get_hash_values(self))

    def __lt__(self, expr):
        r'''Is true if `expr` is a bow contact point and this bow contact point
        is less than `expr`.

        Returns boolean.
        '''
        if isinstance(expr, type(self)):
            return self.contact_point < expr.contact_point
        raise TypeError('unorderable types')

    ### PUBLIC PROPERTIES ###

    @property
    def contact_point(self):
        r'''Gets contact point.

        ::

            >>> indicator = indicatortools.BowContactPoint((1, 4))
            >>> indicator.contact_point
            Multiplier(1, 4)

        Returns multiplier.
        '''
        return self._contact_point

    @property
    def markup(self):
        r'''Gets bow contact point markup.

        ::

            >>> indicator = indicatortools.BowContactPoint((3, 4))
            >>> print(format(indicator.markup, 'lilypond'))
            \markup {
                \vcenter
                    \fraction
                        3
                        4
                }

        '''
        string = r'\vcenter \fraction {} {}'.format(
            self.contact_point.numerator,
            self.contact_point.denominator,
            )
        markup = markuptools.Markup(string)
        return markup



# -*- encoding: utf-8 -*-
from abjad.tools.abctools.AbjadObject import AbjadObject


class Arpeggio(AbjadObject):
    r'''An arpeggio indication.

    ::

        >>> chord = Chord("<c' e' g' c''>4")
        >>> arpeggio = indicatortools.Arpeggio()
        >>> attach(arpeggio, chord)
        >>> show(chord) # doctest: +SKIP

    ..  doctest::

        >>> print(format(chord))
        <c' e' g' c''>4 \arpeggio

    An arpeggio arrow direction can be specified:

    ::

        >>> chord = Chord("<c' e' g' c''>4")
        >>> arpeggio = indicatortools.Arpeggio(direction=Down)
        >>> attach(arpeggio, chord)
        >>> show(chord) # doctest: +SKIP

    ..  doctest::

        >>> print(format(chord))
        \arpeggioArrowDown
        <c' e' g' c''>4 \arpeggio

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_direction',
        )

    ### INITIALIZER ###

    def __init__(self, direction=None):
        assert direction in (Up, Down, Center, None)
        self._direction = direction

    ### SPECIAL METHODS ###

    def __eq__(self, expr):
        r'''Is true when `expr` is an arpeggio indication with a direction
        equal to that of this arpeggio indication. Otherwise false.

        Returns boolean.
        '''
        if isinstance(expr, type(self)):
            if expr.direction == self.direction:
                return True
        return False

    def __hash__(self):
        r'''Hashes arpeggio.

        Required to be explicitely re-defined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(Arpeggio, self).__hash__()

    ### PRIVATE PROPERTIES ###

    @property
    def _lilypond_format(self):
        return r'\arpeggio'

    @property
    def _lilypond_format_bundle(self):
        from abjad.tools import systemtools
        lilypond_format_bundle = systemtools.LilyPondFormatBundle()
        lilypond_format_bundle.right.articulations.append(r'\arpeggio')
        if self.direction in (Up, Down):
            if self.direction is Up:
                command = r'\arpeggioArrowUp'
            else:
                command = r'\arpeggioArrowDown'
            lilypond_format_bundle.before.commands.append(command)
        return lilypond_format_bundle

    ### PUBLIC PROPERTIES ###

    @property
    def direction(self):
        r'''Gets arpeggio arrow direction.

        Return ordinal constant or none.
        '''
        return self._direction



# -*- encoding: utf-8 -*-
from abjad import *


def test_layouttools_set_line_breaks_by_line_duration_in_seconds_ge_01():
    r'''Iterate line-break class instances in expr and
    accumulate duration in seconds.
    Add line break after every total less than or equal to line_duration.
    '''

    staff = Staff()
    staff.append(Measure((2, 8), "c'8 d'8"))
    staff.append(Measure((2, 8), "e'8 f'8"))
    staff.append(Measure((2, 8), "g'8 a'8"))
    staff.append(Measure((2, 8), "b'8 c''8"))
    tempo = Tempo(Duration(1, 8), 44)
    attach(tempo, staff, scope=Staff)
    layouttools.set_line_breaks_by_line_duration_in_seconds_ge(
        staff,
        Duration(6),
        )

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            \tempo 8=44
            {
                \time 2/8
                c'8
                d'8
            }
            {
                e'8
                f'8
                \break
            }
            {
                g'8
                a'8
            }
            {
                b'8
                c''8
            }
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


# -*- encoding: utf-8 -*-
from abjad import *


def test_rhythmmakertools_RestRhythmMaker___call___01():

    maker = rhythmmakertools.RestRhythmMaker()

    divisions = [(5, 16), (3, 8)]
    leaf_lists = maker(divisions)
    leaves = sequencetools.flatten_sequence(leaf_lists)

    measures = scoretools.make_spacer_skip_measures(divisions)
    staff = Staff(measures)
    mutate(staff).replace_measure_contents(leaves)

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 5/16
                r4
                r16
            }
            {
                \time 3/8
                r4.
            }
        }
        '''
        )


# -*- encoding: utf-8 -*-
from abjad import *


def test_datastructuretools_CyclicTree___format___01():

    cyclic_tree = datastructuretools.CyclicPayloadTree([[1, 2, 3], [4, 5]])

    # TODO: make fully recursive to indent at multiple levels
    assert systemtools.TestManager.compare(
        format(cyclic_tree),
        r'''
        datastructuretools.CyclicPayloadTree(
            [
                [1, 2, 3],
                [4, 5],
                ]
            )
        '''
        )


# -*- encoding: utf-8 -*-
import os
from abjad import *
configuration = systemtools.AbjadConfiguration()
ly_path = os.path.join(
    configuration.abjad_directory, 
    'test.ly',
    )
candidate_ly_path = os.path.join(
    configuration.abjad_directory,
    'test.candidate.ly',
    )


def test_agenttools_PersistenceAgent_as_ly_01():
    r'''When candidacy is true, agent persists LilyPond file when no 
    LilyPond file exists.
    '''

    note = Note("c'4")
    with systemtools.FilesystemState(remove=[ly_path, candidate_ly_path]):
        result = persist(note).as_ly(ly_path, candidacy=True)    
        assert os.path.isfile(ly_path)
        assert isinstance(result, tuple)

        
def test_agenttools_PersistenceAgent_as_ly_02():
    r'''When candidacy is true, agent refuses to persist LilyPond file when
    equivalent LilyPond file already exists.
    '''

    note = Note("c'4")
    with systemtools.FilesystemState(remove=[ly_path, candidate_ly_path]):
        result = persist(note).as_ly(ly_path)
        assert isinstance(result, tuple)
        assert os.path.isfile(ly_path)
        result = persist(note).as_ly(ly_path, candidacy=True)    
        assert result == False


def test_agenttools_PersistenceAgent_as_ly_03():
    r'''When candidacy is true, agent persists LilyPond file when
    nonequivalent LilyPond file already exists.
    '''

    note = Note("c'4")
    with systemtools.FilesystemState(remove=[ly_path, candidate_ly_path]):
        with open(ly_path, 'w') as file_pointer:
            file_pointer.write('extra text')
        with open(ly_path, 'r') as file_pointer:
            lines = file_pointer.readlines()
        contents = ''.join(lines)
        assert 'extra text' in contents
        result = persist(note).as_ly(ly_path, candidacy=True)    
        assert isinstance(result, tuple)
        assert os.path.isfile(ly_path)
        with open(ly_path, 'r') as file_pointer:
            lines = file_pointer.readlines()
        contents = ''.join(lines)
        assert 'extra text' not in contents


# -*- encoding: utf-8 -*-
import abc
import collections
import types
from abjad.tools.datastructuretools import TypedTuple
from abjad.tools.topleveltools import new


class Segment(TypedTuple):
    r'''Music-theoretic segment base class.
    '''

    ### CLASS VARIABLES ##

    __slots__ = (
        )

    ### INITIALIZER ###

    def __init__(
        self,
        items=None,
        item_class=None,
        ):
        from abjad.tools import datastructuretools
        from abjad.tools import pitchtools
        prototype = (
            collections.Iterator,
            types.GeneratorType,
            )
        if isinstance(items, str):
            items = items.split()
        elif isinstance(items, prototype):
            items = [item for item in items]
        if item_class is None:
            item_class = self._named_item_class
            if items is not None:
                if isinstance(items, datastructuretools.TypedCollection) and \
                    issubclass(items.item_class, self._parent_item_class):
                    item_class = items.item_class
                elif len(items):
                    if isinstance(items, collections.Set):
                        items = tuple(items)
                    if isinstance(items[0], str):
                        item_class = self._named_item_class
                    elif isinstance(items[0], (int, float)):
                        item_class = self._numbered_item_class
                    elif isinstance(items[0], self._parent_item_class):
                        item_class = type(items[0])
        assert issubclass(item_class, self._parent_item_class)
        TypedTuple.__init__(
            self,
            items=items,
            item_class=item_class,
            )

    ### SPECIAL METHODS ###

    def __str__(self):
        r'''String representation of segment.

        Returns string.
        '''
        parts = [str(x) for x in self]
        return '<{}>'.format(', '.join(parts))

    ### PRIVATE PROPERTIES ###

    @abc.abstractproperty
    def _named_item_class(self):
        raise NotImplementedError

    @abc.abstractproperty
    def _numbered_item_class(self):
        raise NotImplementedError

    @abc.abstractproperty
    def _parent_item_class(self):
        raise NotImplementedError

    @property
    def _repr_specification(self):
        items = []
        if self.item_class.__name__.startswith('Named'):
            items = [str(x) for x in self]
        elif hasattr(self.item_class, 'pitch_number'):
            items = [x.pitch_number for x in self]
        elif hasattr(self.item_class, 'pitch_class_number'):
            items = [x.pitch_class_number for x in self]
        elif hasattr(self.item_class, '__abs__'):
            items = [abs(x) for x in self]
        else:
            raise ValueError
        return new(
            self._storage_format_specification,
            is_indented=False,
            keyword_argument_names=(),
            positional_argument_values=(
                items,
                ),
            )

    ### PUBLIC METHODS ###

    @abc.abstractmethod
    def from_selection(
        cls,
        selection,
        item_class=None,
        ):
        r'''Makes segment from `selection`.

        Returns new segment.
        '''
        raise NotImplementedError

    ### PUBLIC PROPERTIES ###

    @abc.abstractproperty
    def has_duplicates(self):
        r'''Is true when segment has duplicates. Otherwise false.

        Returns boolean.
        '''
        raise NotImplementedError


# -*- encoding: utf-8 -*-
import pytest
from abjad import *


def test_mathtools_partition_integer_by_ratio_01():
    r'''Partition integer n according to ratio.
    '''

    t = mathtools.partition_integer_by_ratio(10, [1])
    assert t == [10]

    t = mathtools.partition_integer_by_ratio(10, [1, 1])
    assert t == [5, 5]

    t = mathtools.partition_integer_by_ratio(10, [1, -1, -1])
    assert t == [3, -4, -3]

    t = mathtools.partition_integer_by_ratio(-10, [1, 1, 1, 1])
    assert t == [-3, -2, -3, -2]

    t = mathtools.partition_integer_by_ratio(-10, [1, 1, 1, 1, 1])
    assert t == [-2, -2, -2, -2, -2]


def test_mathtools_partition_integer_by_ratio_02():
    r'''Partition integer n according to ratio.
    '''

    partition = mathtools.partition_integer_by_ratio(10, [1, 2])
    assert partition == [3, 7]

    partition = mathtools.partition_integer_by_ratio(10, [3, 1])
    assert partition == [8, 2]

    partition = mathtools.partition_integer_by_ratio(-10, [-3, 2])
    assert partition == [6, -4]


def test_mathtools_partition_integer_by_ratio_03():
    r'''Raise type error on noninteger n.
    '''

    statement = "mathtools.partition_integer_by_ratio('foo', [1, 1, 1])"
    assert pytest.raises(TypeError, statement)


# -*- encoding: utf-8 -*-
import pytest
from abjad.tools.sievetools.ResidueClass import ResidueClass


RC = ResidueClass

def test_sievetools_ResidueClass_01():
    r'''modulo must be positive.
    '''

    pytest.raises(ValueError, 't = RC(0, 1)')


def test_sievetools_ResidueClass_02():
    r'''residue must be non-negative and < modulo.
    '''

    pytest.raises(ValueError, 't = RC(2, 13)')
    pytest.raises(ValueError, 't = RC(2, 2)')
    pytest.raises(ValueError, 't = RC(2, -1)')


def test_sievetools_ResidueClass_03():
    r'''modulo may be 1.
    '''

    rc = RC(1, 0)

    assert rc.modulo == 1
    assert rc.residue == 0


def test_sievetools_ResidueClass_04():

    rc = RC(2, 0)

    assert rc.modulo == 2
    assert rc.residue == 0
    assert rc.get_congruent_bases(4) == [0,2,4]
    assert rc.get_boolean_train(4) == [1,0,1,0]


def test_sievetools_ResidueClass_05():

    rc = RC(2, 1)

    assert rc.modulo == 2
    assert rc.residue == 1
    assert rc.get_congruent_bases(5) == [1,3,5]
    assert rc.get_boolean_train(4) == [0,1,0,1]


def test_sievetools_ResidueClass_06():

    rc = RC(3, 0)

    assert rc.modulo == 3
    assert rc.residue == 0
    assert rc.get_congruent_bases(6) == [0, 3, 6]
    assert rc.get_boolean_train(6) == [1,0,0,1,0,0]


def test_sievetools_ResidueClass_07():

    rc = RC(3, 1)

    assert rc.modulo == 3
    assert rc.residue == 1
    assert rc.get_congruent_bases(7) == [1, 4, 7]
    assert rc.get_boolean_train(6) == [0,1,0,0,1,0]


def test_sievetools_ResidueClass_08():
    r'''get_congruent_bases() and get_boolean_train() must take
    range parameters.'''

    rc = RC(1, 0)

    pytest.raises(Exception, 'rc = rc. get_congruent_bases()')
    pytest.raises(Exception, 'rc = rc. get_boolean_train()')

    assert isinstance(rc.get_congruent_bases(12), list)
    assert isinstance(rc.get_boolean_train(12), list)

    assert rc.get_congruent_bases(99) == list(range(100))
    assert rc.get_congruent_bases(-10, 99) == list(range(-10, 100))

    assert rc.get_boolean_train(12) == [1] * 12
    assert rc.get_boolean_train(-2, 12) == [1] * 14



# -*- encoding: utf-8 -*-
import collections
from abjad.tools.datastructuretools.TypedCollection import TypedCollection


class TypedList(TypedCollection):
    r'''A typed list.

    Ordered collection of objects, which optionally coerces its contents
    to the same type:

    ::

        >>> object_collection = datastructuretools.TypedList()
        >>> object_collection.append(23)
        >>> object_collection.append('foo')
        >>> object_collection.append(False)
        >>> object_collection.append((1, 2, 3))
        >>> object_collection.append(3.14159)

    ::

        >>> print(format(object_collection))
        datastructuretools.TypedList(
            [
                23,
                'foo',
                False,
                (1, 2, 3),
                3.14159,
                ]
            )

    ::

        >>> pitch_collection = datastructuretools.TypedList(
        ...     item_class=NamedPitch)
        >>> pitch_collection.append(0)
        >>> pitch_collection.append("d'")
        >>> pitch_collection.append(('e', 4))
        >>> pitch_collection.append(NamedPitch("f'"))

    ::

        >>> print(format(pitch_collection))
        datastructuretools.TypedList(
            [
                pitchtools.NamedPitch("c'"),
                pitchtools.NamedPitch("d'"),
                pitchtools.NamedPitch("e'"),
                pitchtools.NamedPitch("f'"),
                ],
            item_class=pitchtools.NamedPitch,
            )

    Implements the list interface.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_keep_sorted',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        items=None,
        item_class=None,
        keep_sorted=None,
        ):
        TypedCollection.__init__(
            self,
            item_class=item_class,
            items=items,
            )
        self._collection = []
        if keep_sorted:
            self._keep_sorted = True
        else:
            self._keep_sorted = None
        items = items or []
        the_items = []
        for item in items:
            the_items.append(self._item_callable(item))
        self.extend(the_items)

    ### SPECIAL METHODS ###

    def __delitem__(self, i):
        r'''Aliases list.__delitem__().

        Returns none.
        '''
        self._on_removal(self._collection[i])
        del(self._collection[i])

    def __getitem__(self, i):
        r'''Aliases list.__getitem__().

        Returns item.
        '''
        return self._collection[i]

    def __iadd__(self, expr):
        r'''Changes items in `expr` to items and extends.

        ::

            >>> dynamic_collection = datastructuretools.TypedList(
            ...     item_class=Dynamic)
            >>> dynamic_collection.append('ppp')
            >>> dynamic_collection += ['p', 'mp', 'mf', 'fff']

        ::

            >>> print(format(dynamic_collection))
            datastructuretools.TypedList(
                [
                    indicatortools.Dynamic(
                        name='ppp',
                        ),
                    indicatortools.Dynamic(
                        name='p',
                        ),
                    indicatortools.Dynamic(
                        name='mp',
                        ),
                    indicatortools.Dynamic(
                        name='mf',
                        ),
                    indicatortools.Dynamic(
                        name='fff',
                        ),
                    ],
                item_class=indicatortools.Dynamic,
                )

        Returns collection.
        '''
        self.extend(expr)
        return self

    def __reversed__(self):
        r'''Aliases list.__reversed__().

        Returns generator.
        '''
        return self._collection.__reversed__()

    def __setitem__(self, i, expr):
        r'''Changes items in `expr` to items and sets.

        ::

            >>> pitch_collection[-1] = 'gqs,'
            >>> print(format(pitch_collection))
            datastructuretools.TypedList(
                [
                    pitchtools.NamedPitch("c'"),
                    pitchtools.NamedPitch("d'"),
                    pitchtools.NamedPitch("e'"),
                    pitchtools.NamedPitch('gqs,'),
                    ],
                item_class=pitchtools.NamedPitch,
                )

        ::

            >>> pitch_collection[-1:] = ["f'", "g'", "a'", "b'", "c''"]
            >>> print(format(pitch_collection))
            datastructuretools.TypedList(
                [
                    pitchtools.NamedPitch("c'"),
                    pitchtools.NamedPitch("d'"),
                    pitchtools.NamedPitch("e'"),
                    pitchtools.NamedPitch("f'"),
                    pitchtools.NamedPitch("g'"),
                    pitchtools.NamedPitch("a'"),
                    pitchtools.NamedPitch("b'"),
                    pitchtools.NamedPitch("c''"),
                    ],
                item_class=pitchtools.NamedPitch,
                )

        Returns none.
        '''
        if isinstance(i, int):
            new_item = self._item_callable(expr)
            old_item = self._collection[i]
            self._on_removal(old_item)
            self._on_insertion(new_item)
            self._collection[i] = new_item
        elif isinstance(i, slice):
            new_items = [self._item_callable(item) for item in expr]
            old_items = self._collection[i]
            for old_item in old_items:
                self._on_removal(old_item)
            for new_item in new_items:
                self._on_insertion(new_item)
            self._collection[i] = new_items
        if self.keep_sorted:
            self.sort()

    ### PUBLIC METHODS ###

    def append(self, item):
        r'''Changes `item` to item and appends.

        ::

            >>> integer_collection = datastructuretools.TypedList(
            ...     item_class=int)
            >>> integer_collection[:]
            []

        ::

            >>> integer_collection.append('1')
            >>> integer_collection.append(2)
            >>> integer_collection.append(3.4)
            >>> integer_collection[:]
            [1, 2, 3]

        Returns none.
        '''
        item = self._item_callable(item)
        self._on_insertion(item)
        self._collection.append(item)
        if self.keep_sorted:
            self.sort()

    def count(self, item):
        r'''Changes `item` to item and returns count.

        ::

            >>> integer_collection = datastructuretools.TypedList(
            ...     items=[0, 0., '0', 99],
            ...     item_class=int)
            >>> integer_collection[:]
            [0, 0, 0, 99]

        ::

            >>> integer_collection.count(0)
            3

        Returns count.
        '''
        item = self._item_callable(item)
        return self._collection.count(item)

    def extend(self, items):
        r'''Changes `items` to items and extends.

        ::

            >>> integer_collection = datastructuretools.TypedList(
            ...     item_class=int)
            >>> integer_collection.extend(('0', 1.0, 2, 3.14159))
            >>> integer_collection[:]
            [0, 1, 2, 3]

        Returns none.
        '''
        for item in items:
            self.append(item)
        if self.keep_sorted:
            self.sort()

    def index(self, item):
        r'''Changes `item` to item and returns index.

        ::

            >>> pitch_collection = datastructuretools.TypedList(
            ...     items=('cqf', "as'", 'b,', 'dss'),
            ...     item_class=NamedPitch)
            >>> pitch_collection.index("as'")
            1

        Returns index.
        '''
        item = self._item_callable(item)
        return self._collection.index(item)

    def insert(self, i, item):
        r'''Changes `item` to item and inserts.

        ::

            >>> integer_collection = datastructuretools.TypedList(
            ...     item_class=int)
            >>> integer_collection.extend(('1', 2, 4.3))
            >>> integer_collection[:]
            [1, 2, 4]

        ::

            >>> integer_collection.insert(0, '0')
            >>> integer_collection[:]
            [0, 1, 2, 4]

        ::

            >>> integer_collection.insert(1, '9')
            >>> integer_collection[:]
            [0, 9, 1, 2, 4]

        Returns none.
        '''
        item = self._item_callable(item)
        self._on_insertion(item)
        result = self._collection.insert(i, item)
        if self.keep_sorted:
            self.sort()
        return result

    def pop(self, i=-1):
        r'''Aliases list.pop().
        '''
        result = self._collection.pop(i)
        self._on_removal(result)
        if self.keep_sorted:
            self.sort()
        return result

    def remove(self, item):
        r'''Changes `item` to item and removes.

        ::

            >>> integer_collection = datastructuretools.TypedList(
            ...     item_class=int)
            >>> integer_collection.extend(('0', 1.0, 2, 3.14159))
            >>> integer_collection[:]
            [0, 1, 2, 3]

        ::

            >>> integer_collection.remove('1')
            >>> integer_collection[:]
            [0, 2, 3]

        Returns none.
        '''
        item = self._item_callable(item)
        index = self._collection.index(item)
        item = self._collection[index]
        self._on_removal(item)
        del(self._collection[index])
        if self.keep_sorted:
            self.sort()

    def reverse(self):
        r'''Aliases list.reverse().
        '''
        self._collection.reverse()

    def sort(self, cmp=None, key=None, reverse=False):
        r'''Aliases list.sort().
        '''
        if cmp is not None:
            def cmp_to_key(comparator):
                r'''Convert a `cmp` function into a `key` function for use
                with `sort()`.
                '''
                class CmpToKey(object):

                    def __init__(self, object_):
                        self.object_ = object_

                    def __lt__(self, other):
                        return comparator(self.object_, other.object_) < 0

                    def __gt__(self, other):
                        return comparator(self.object_, other.object_) > 0

                    def __eq__(self, other):
                        return comparator(self.object_, other.object_) == 0

                    def __le__(self, other):
                        return comparator(self.object_, other.object_) <= 0

                    def __ge__(self, other):
                        return comparator(self.object_, other.object_) >= 0

                    def __ne__(self, other):
                        return comparator(self.object_, other.object_) != 0

                return CmpToKey
            key = cmp_to_key(cmp)
        self._collection.sort(key=key, reverse=reverse)

    ### PUBLIC PROPERTIES ###

    @property
    def keep_sorted(self):
        r'''Sorts collection on mutation if true.
        '''
        return self._keep_sorted

    @keep_sorted.setter
    def keep_sorted(self, expr):
        assert isinstance(expr, (bool, type(None)))
        self._keep_sorted = expr


collections.MutableSequence.register(TypedList)



# -*- encoding: utf-8 -*-
from abjad.tools.abctools.AbjadObject import AbjadObject


class LilyPondNameManager(AbjadObject):
    r'''Base class from which LilyPond grob and setting managers inherit.
    '''

    ### SPECIAL METHODS ###

    def __eq__(self, arg):
        r'''Is true when `arg` is a LilyPond name manager with attribute pairs
        equal to those of this LilyPond name manager. Otherwise false.

        Returns boolean.
        '''
        if isinstance(arg, type(self)):
            return self._get_attribute_pairs() == arg._get_attribute_pairs()
        return False

    def __getstate__(self):
        r'''Gets object state.
        '''
        import copy
        return copy.deepcopy(vars(self))

    def __hash__(self):
        r'''Hashes LilyPond name manager.

        Required to be explicitely re-defined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(LilyPondNameManager, self).__hash__()

    def __repr__(self):
        r'''Gets interpreter representation of LilyPond name manager.

        Returns string.
        '''
        body_string = ''
        strings = self._get_skeleton_strings()
        if strings:
            prefix = getattr(self, 'skeleton_string_prefix', '')
            strings = [x.replace(prefix, '') for x in strings]
            body_string = ', '.join(strings)
        return '{}({})'.format(type(self).__name__, body_string)

    def __setstate__(self, state):
        r'''Sets object state.
        '''
        for key, value in state.items():
            self.__dict__[key] = value

    ### PRIVATE METHODS ###

    def _get_attribute_pairs(self):
        return tuple(sorted(vars(self).items()))

    def _get_skeleton_strings(self):
        result = []
        for attribute_name, attribute_value in self._get_attribute_pairs():
            string = '{} = {}'.format(attribute_name, repr(attribute_value))
            result.append(string)
        return result



# -*- encoding: utf-8 -*-


def timespan_2_delays_timespan_1(timespan_1=None, timespan_2=None, hold=False):
    r'''Makes time relation indicating that `timespan_2` delays `timespan_1`.

    ::

        >>> relation = timespantools.timespan_2_delays_timespan_1()
        >>> print(format(relation))
        timespantools.TimespanTimespanTimeRelation(
            inequality=timespantools.CompoundInequality(
                [
                    timespantools.SimpleInequality('timespan_2.start_offset <= timespan_1.start_offset'),
                    timespantools.SimpleInequality('timespan_1.start_offset < timespan_2.stop_offset'),
                    ],
                logical_operator='and',
                ),
            )

    Returns time relation or boolean.
    '''
    from abjad.tools import timespantools

    inequality = timespantools.CompoundInequality([
        'timespan_2.start_offset <= timespan_1.start_offset',
        'timespan_1.start_offset < timespan_2.stop_offset',
        ])

    time_relation = timespantools.TimespanTimespanTimeRelation(
        inequality,
        timespan_1=timespan_1,
        timespan_2=timespan_2,
        )

    if time_relation.is_fully_loaded and not hold:
        return time_relation()
    else:
        return time_relation



# -*- encoding: utf-8 -*-
from abjad import *
from abjad.tools import tonalanalysistools


def test_tonalanalysistools_ChordSuspension__initialize_by_pair_01():

    chord_suspension = tonalanalysistools.ChordSuspension((4, 3))

    assert chord_suspension.start == tonalanalysistools.ScaleDegree(4)
    assert chord_suspension.stop == tonalanalysistools.ScaleDegree(3)


# -*- encoding: utf-8 -*-
import re


lowercamelcase_regex = re.compile('^([a-z,0-9]+([A-Z,0-9]+[a-z,0-9]*)*)?$')

def is_lower_camel_case(expr):
    r'''Is true when `expr` is a string and is lowercamelcase.

    ..  container:: example

        ::

            >>> stringtools.is_lower_camel_case('fooBar')
            True

    Otherwise false:

    ..  container:: example

        ::

            >>> stringtools.is_lower_camel_case('FooBar')
            False

    Returns boolean.
    '''

    if not isinstance(expr, str):
        return False

    return bool(lowercamelcase_regex.match(expr))


# -*- encoding: utf-8 -*-
import collections
from abjad.tools.datastructuretools.TypedCollection import TypedCollection
from abjad.tools.topleveltools import new


class TypedTuple(TypedCollection):
    r'''A typed tuple.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### INITIALIZER ###

    def __init__(self, items=None, item_class=None):
        TypedCollection.__init__(self,
            item_class=item_class,
            items=items,
            )
        items = items or []
        self._collection = tuple(
            self._item_callable(item) for item in items)

    ### SPECIAL METHODS ###

    def __add__(self, expr):
        r'''Adds typed tuple to `expr`.

        Returns new typed tuple.
        '''
        if isinstance(expr, type(self)):
            items = expr._collection
            return new(self, items=self._collection[:] + items)
        elif isinstance(expr, type(self._collection)):
            items = expr[:]
            return new(self, items=self._collection[:] + items)
        raise NotImplementedError

    def __contains__(self, item):
        r'''Change `item` to item and return true if item exists in
        collection.

        Returns none.
        '''
        try:
            item = self._item_callable(item)
        except ValueError:
            return False
        return self._collection.__contains__(item)

    def __getitem__(self, i):
        '''Gets `i` from type tuple.

        Returns item.
        '''
        if type(i) == slice:
            return self.__getslice__(i.start, i.stop)
        return self._collection[i]

    def __getslice__(self, start, stop):
        r'''Gets slice from `start` to `stop` in typed tuple.

        Returns new typed tuple.
        '''
        return new(self, items=self._collection[start:stop])

    def __hash__(self):
        r'''Hashes typed tuple.

        Returns integer.
        '''
        return hash((
            type(self),
            self._collection,
            self.item_class,
            ))

    def __mul__(self, expr):
        r'''Multiplies typed tuple by `expr`.

        Returns new typed tuple.
        '''
        items = self._collection * expr
        return new(self, items=items)

    def __radd__(self, expr):
        r'''Right-adds `expr` to typed tuple.
        '''
        items = expr + self._collection
        return new(self, items=items)

    def __rmul__(self, expr):
        r'''Multiplies `expr` by typed tuple.

        Returns new typed tuple.
        '''
        return self.__mul__(expr)

    ### PUBLIC METHODS ###

    def count(self, item):
        r'''Changes `item` to item.

        Returns count in collection.
        '''
        item = self._item_callable(item)
        return self._collection.count(item)

    def index(self, item):
        r'''Changes `item` to item.

        Returns index in collection.
        '''
        item = self._item_callable(item)
        return self._collection.index(item)


collections.Sequence.register(TypedTuple)


# -*- encoding: utf-8 -*-
import copy
from abjad import *


def test_quantizationtools_QGrid___copy___01():

    q_grid = quantizationtools.QGrid()
    copied = copy.copy(q_grid)

    assert q_grid == copied
    assert q_grid is not copied

    assert q_grid.root_node == copied.root_node
    assert q_grid.root_node is not copied.root_node

    assert q_grid.next_downbeat == copied.next_downbeat
    assert q_grid.next_downbeat is not copied.next_downbeat


# -*- encoding: utf-8 -*-
from abjad.tools import scoretools
from abjad.tools.topleveltools import detach
from abjad.tools.topleveltools import iterate


def remove_markup_from_leaves_in_expr(expr):
    r'''Remove markup from leaves in `expr`:

    ::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> labeltools.label_leaves_in_expr_with_pitch_class_numbers(staff)
        >>> print(format(staff))
        \new Staff {
            c'8 _ \markup { \small 0 }
            d'8 _ \markup { \small 2 }
            e'8 _ \markup { \small 4 }
            f'8 _ \markup { \small 5 }
        }

    ::

        >>> show(staff) # doctest: +SKIP

    ::

        >>> labeltools.remove_markup_from_leaves_in_expr(staff)
        >>> print(format(staff))
        \new Staff {
            c'8
            d'8
            e'8
            f'8
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Returns none.
    '''
    from abjad.tools import markuptools

    for leaf in iterate(expr).by_class(scoretools.Leaf):
        detach(markuptools.Markup, leaf)



# -*- encoding: utf-8 -*-
from abjad.tools.datastructuretools import TypedList


class SelectionInventory(TypedList):
    r'''An inventory of component selections.

    ::

        >>> inventory = selectiontools.SelectionInventory()

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### PRIVATE PROPERTIES ###

    @property
    def _item_callable(self):
        from abjad.tools import selectiontools
        return selectiontools.SliceSelection


# -*- encoding: utf-8 -*-
from abjad import *


def test_markuptools_Markup__get_format_pieces_01():

    circle = markuptools.MarkupCommand('draw-circle', 2.5, 0.1, False)
    square = markuptools.MarkupCommand('rounded-box', 'hello?')
    line = markuptools.MarkupCommand('line', [square, 'wow!'])
    markup = markuptools.Markup(('X', circle, 'Y', line, 'Z'), direction=Up)

    assert markup._get_format_pieces() == [
        '^ \\markup {',
        '\tX',
        '\t\\draw-circle',
        '\t\t#2.5',
        '\t\t#0.1',
        '\t\t##f',
        '\tY',
        '\t\\line',
        '\t\t{',
        '\t\t\t\\rounded-box',
        '\t\t\t\thello?',
        '\t\t\twow!',
        '\t\t}',
        '\tZ',
        '\t}'
    ]


# -*- encoding: utf-8 -*-
import os
from abjad.tools.abctools import ContextManager


class TemporaryDirectoryChange(ContextManager):
    r'''A context manager for temporarily changing the current working
    directory.
    '''

    ### INITIALIZER ###

    def __init__(self, directory=None):
        if directory is None:
            pass
        elif os.path.isdir(directory):
            pass
        elif os.path.isfile(directory):
            directory = os.path.dirname(directory)
        self._directory = directory

    ### SPECIAL METHODS ###

    def __enter__(self):
        r'''Enters context manager and changes to `directory`.
        '''
        self._original_directory = os.getcwd()
        if self._directory is not None:
            os.chdir(self.directory)
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        r'''Exits context manager and returns to original working directory.
        '''
        if self._directory is not None:
            os.chdir(self._original_directory)

    def __repr__(self):
        r'''Gets interpreter representation of context manager.

        Returns string.
        '''
        return '<{}()>'.format(type(self).__name__)

    ### PUBLIC PROPERTIES ###

    @property
    def directory(self):
        r'''Gets temporary directory of context manager.

        Returns string.
        '''
        return self._directory


# -*- encoding: utf-8 -*-


def transpose_pitch_expr_into_pitch_range(pitch_expr, pitch_range):
    '''Transpose `pitch_expr` into `pitch_range`:

    ::

        >>> pitches = [-2, -1, 13, 14]
        >>> range_ = pitchtools.PitchRange('[C4, C5]')
        >>> pitchtools.transpose_pitch_expr_into_pitch_range(pitches, range_)
        [10, 11, 1, 2]

    Returns new `pitch_expr` object.
    '''

    try:
        transposed_pitch_carriers = [
            _transpose_pitch_carrier_into_pitch_range(x, pitch_range) for x in pitch_expr]
        return type(pitch_expr)(transposed_pitch_carriers)
    except TypeError:
        return _transpose_pitch_carrier_into_pitch_range(pitch_expr, pitch_range)


# TODO: make public? make nested helper?
def _transpose_pitch_carrier_into_pitch_range(pitch_carrier, pitch_range):
    from abjad.tools import pitchtools

    while pitch_carrier < pitch_range:
        pitch_carrier = pitchtools.transpose_pitch_carrier_by_interval(pitch_carrier, 12)
    while pitch_range < pitch_carrier:
        pitch_carrier = pitchtools.transpose_pitch_carrier_by_interval(pitch_carrier, -12)
    if pitch_carrier not in pitch_range:
        message = 'can not transpose pitch carrier {} into pitch range.'
        message = message.format(pitch_carrier)
        raise ValueError(message)
    return pitch_carrier


# -*- encoding: utf-8 -*-
from abjad import *


def test_scoretools_NoteHeadInventory_extend_01():
    r'''Extend chord with tweaked note heads.
    '''

    chord = Chord("<ef'>4")
    note_heads = []
    note_head = scoretools.NoteHead("cs''")
    note_head.tweak.color = 'blue'
    note_heads.append(note_head)
    note_head = scoretools.NoteHead("f''")
    note_head.tweak.color = 'green'
    note_heads.append(note_head)
    chord.note_heads.extend(note_heads)

    assert systemtools.TestManager.compare(
        chord,
        r'''
        <
            ef'
            \tweak #'color #blue
            cs''
            \tweak #'color #green
            f''
        >4
        '''
        )

    assert inspect_(chord).is_well_formed()


# -*- encoding: utf-8 -*-


def iterate_sequence_nwise(sequence, n=2, cyclic=False, wrapped=False):
    '''Iterates elements in `sequence` `n` at a time.

    ..  container:: example

        Iterates sequence by pairs:

        ::

            >>> for x in sequencetools.iterate_sequence_nwise(range(10)):
            ...     x
            ...
            (0, 1)
            (1, 2)
            (2, 3)
            (3, 4)
            (4, 5)
            (5, 6)
            (6, 7)
            (7, 8)
            (8, 9)

    ..  container:: example

        Iterates sequence by triples:

        ::

            >>> for x in sequencetools.iterate_sequence_nwise(range(10), n=3):
            ...     x
            ...
            (0, 1, 2)
            (1, 2, 3)
            (2, 3, 4)
            (3, 4, 5)
            (4, 5, 6)
            (5, 6, 7)
            (6, 7, 8)
            (7, 8, 9)


    ..  container:: example

        Iterates sequence by pairs. Wraps around at end:

        ::

            >>> for x in sequencetools.iterate_sequence_nwise(
            ...     range(10), n=2, wrapped=True):
            ...     x
            ...
            (0, 1)
            (1, 2)
            (2, 3)
            (3, 4)
            (4, 5)
            (5, 6)
            (6, 7)
            (7, 8)
            (8, 9)
            (9, 0)

    ..  container:: example

        Iterates sequence by triples. Wraps around at end:

        ::

            >>> for x in sequencetools.iterate_sequence_nwise(
            ...     range(10), n=3, wrapped=True):
            ...     x
            ...
            (0, 1, 2)
            (1, 2, 3)
            (2, 3, 4)
            (3, 4, 5)
            (4, 5, 6)
            (5, 6, 7)
            (6, 7, 8)
            (7, 8, 9)
            (8, 9, 0)
            (9, 0, 1)

    ..  container:: example

        Iterates sequence by pairs. Cycles indefinitely:

        ::

            >>> pairs = sequencetools.iterate_sequence_nwise(
            ...     range(10), n=2, cyclic=True)
            >>> for _ in range(15):
            ...     next(pairs)
            (0, 1)
            (1, 2)
            (2, 3)
            (3, 4)
            (4, 5)
            (5, 6)
            (6, 7)
            (7, 8)
            (8, 9)
            (9, 0)
            (0, 1)
            (1, 2)
            (2, 3)
            (3, 4)
            (4, 5)

        Returns infinite generator.

    ..  container:: example

        Iterates sequence by triples. Cycles indefinitely:

        ::

            >>> triples = sequencetools.iterate_sequence_nwise(
            ...     range(10), n=3, cyclic=True)
            ...
            >>> for _ in range(15):
            ...     next(triples)
            (0, 1, 2)
            (1, 2, 3)
            (2, 3, 4)
            (3, 4, 5)
            (4, 5, 6)
            (5, 6, 7)
            (6, 7, 8)
            (7, 8, 9)
            (8, 9, 0)
            (9, 0, 1)
            (0, 1, 2)
            (1, 2, 3)
            (2, 3, 4)
            (3, 4, 5)
            (4, 5, 6)

        Returns infinite generator.

    Ignores ``wrapped`` when ``cyclic=True``.

    Returns generator.
    '''

    if cyclic:
        element_buffer = []
        long_enough = False
        for element in sequence:
            element_buffer.append(element)
            if not long_enough:
                if n <= len(element_buffer):
                    long_enough = True
            if long_enough:
                yield tuple(element_buffer[-n:])
        len_sequence = len(element_buffer)
        current = len_sequence - n + 1
        while True:
            output = []
            for local_offset in range(n):
                index = (current + local_offset) % len_sequence
                output.append(element_buffer[index])
            yield tuple(output)
            current += 1
            current %= len_sequence
    elif wrapped:
        first_n_minus_1 = []
        element_buffer = []
        for element in sequence:
            element_buffer.append(element)
            if len(element_buffer) == n:
                yield tuple(element_buffer)
                element_buffer.pop(0)
            if len(first_n_minus_1) < n - 1:
                first_n_minus_1.append(element)
        element_buffer = element_buffer + first_n_minus_1
        if element_buffer:
            for x in range(n - 1):
                yield tuple(element_buffer[x:x+n])
    else:
        element_buffer = []
        for element in sequence:
            element_buffer.append(element)
            if len(element_buffer) == n:
                yield tuple(element_buffer)
                element_buffer.pop(0)



# -*- encoding: utf-8 -*-


def iterate_sequence_boustrophedon(sequence, duplicates=False):
    '''Iterates `sequence` boustrophedon.

    ..  container:: example

        Iterates `sequence` first forward and then backward.
        Duplicates neither first nor last elements:

        ::

            >>> sequence = [1, 2, 3, 4, 5]
            >>> generator = sequencetools.iterate_sequence_boustrophedon(
            ...     sequence, duplicates=False)
            >>> list(generator)
            [1, 2, 3, 4, 5, 4, 3, 2]

    ..  container:: example

        Iterates `sequence` first forward and then backward.
        Duplicates both first and last elements:

        ::

            >>> sequence = [1, 2, 3, 4, 5]
            >>> generator = sequencetools.iterate_sequence_boustrophedon(
            ...     sequence, duplicates=True)
            >>> list(generator)
            [1, 2, 3, 4, 5, 5, 4, 3, 2, 1]

    Returns generator.
    '''

    if duplicates:
        sequence_copy = []
        for x in sequence:
            yield x
            sequence_copy.append(x)
        for x in reversed(sequence_copy):
            yield x
    else:
        sequence_copy = []
        for x in sequence:
            yield x
            sequence_copy.append(x)
        for x in reversed(sequence_copy[1:-1]):
            yield x


# -*- encoding: utf-8 -*-
import pytest
from abjad import *


def test_pitchtools_PitchArrayCell_next_01():

    array = pitchtools.PitchArray([[1, 2, 1], [2, 1, 1]])

    '''
    [] [      ] []
    [      ] [] []
    '''

    assert getattr(array[0][1], 'next') is array[0][3]


def test_pitchtools_PitchArrayCell_next_02():

    array = pitchtools.PitchArray([[1, 2, 1], [2, 1, 1]])

    '''
    [] [      ] []
    [      ] [] []
    '''

    assert pytest.raises(IndexError, 'array[0][-1].next')


def test_pitchtools_PitchArrayCell_next_03():

    cell = pitchtools.PitchArrayCell([NamedPitch(1)])

    assert pytest.raises(IndexError, 'cell.next')



# -*- encoding: utf-8 -*-
from abjad.tools import mathtools


def spell_numbered_interval_number(
    named_interval_number,
    numbered_interval_number,
    ):
    '''Spell `numbered_interval_number` according to `named_interval_number`:

    ::

        >>> pitchtools.spell_numbered_interval_number(2, 1)
        NamedInterval('+m2')

    Returns named interval.
    '''
    from abjad.tools import pitchtools

    if not isinstance(numbered_interval_number, int):
        message = 'can not determine diatonic interval from float.'
        raise TypeError(message)

    direction_number = mathtools.sign(numbered_interval_number)

    if named_interval_number == 1:
        if numbered_interval_number % 12 == 11:
            quality_string = 'augmented'
        elif numbered_interval_number % 12 == 0:
            quality_string = 'perfect'
        elif numbered_interval_number % 12 == 1:
            quality_string = 'augmented'
        if not direction_number == 0:
            named_interval_number *= direction_number
        named_interval = pitchtools.NamedInterval(
            quality_string, named_interval_number)
        return named_interval

    named_interval_class_number = named_interval_number % 7
    numbered_interval_class_number = abs(numbered_interval_number) % 12

    if named_interval_class_number == 0:
        if numbered_interval_class_number == 9:
            quality_string = 'diminished'
        elif numbered_interval_class_number == 10:
            quality_string = 'minor'
        elif numbered_interval_class_number == 11:
            quality_string = 'major'
        elif numbered_interval_class_number == 0:
            quality_string = 'augmented'
    elif named_interval_class_number == 1:
        if numbered_interval_class_number == 11:
            quality_string = 'diminished'
        elif numbered_interval_class_number == 0:
            quality_string = 'perfect'
        elif numbered_interval_class_number == 1:
            quality_string = 'augmented'
    elif named_interval_class_number == 2:
        if numbered_interval_class_number == 0:
            quality_string = 'diminished'
        elif numbered_interval_class_number == 1:
            quality_string = 'minor'
        elif numbered_interval_class_number == 2:
            quality_string = 'major'
        elif numbered_interval_class_number == 3:
            quality_string = 'augmented'
    elif named_interval_class_number == 3:
        if numbered_interval_class_number == 2:
            quality_string = 'diminished'
        elif numbered_interval_class_number == 3:
            quality_string = 'minor'
        elif numbered_interval_class_number == 4:
            quality_string = 'major'
        elif numbered_interval_class_number == 5:
            quality_string = 'augmented'
    elif named_interval_class_number == 4:
        if numbered_interval_class_number == 4:
            quality_string = 'diminished'
        elif numbered_interval_class_number == 5:
            quality_string = 'perfect'
        elif numbered_interval_class_number == 6:
            quality_string = 'augmented'
    elif named_interval_class_number == 5:
        if numbered_interval_class_number == 6:
            quality_string = 'diminished'
        elif numbered_interval_class_number == 7:
            quality_string = 'perfect'
        elif numbered_interval_class_number == 8:
            quality_string = 'augmented'
    elif named_interval_class_number == 6:
        if numbered_interval_class_number == 7:
            quality_string = 'diminished'
        elif numbered_interval_class_number == 8:
            quality_string = 'minor'
        elif numbered_interval_class_number == 9:
            quality_string = 'major'
        elif numbered_interval_class_number == 10:
            quality_string = 'augmented'
    elif named_interval_class_number == 7:
        if numbered_interval_class_number == 9:
            quality_string = 'diminished'
        elif numbered_interval_class_number == 10:
            quality_string = 'minor'
        elif numbered_interval_class_number == 11:
            quality_string = 'major'
        elif numbered_interval_class_number == 0:
            quality_string = 'augmented'
    elif named_interval_class_number == 8:
        if numbered_interval_class_number == 11:
            quality_string = 'diminished'
        elif numbered_interval_class_number == 0:
            quality_string = 'perfect'
        elif numbered_interval_class_number == 1:
            quality_string = 'augmented'

    if not direction_number == 0:
        named_interval_number *= direction_number

    named_interval = pitchtools.NamedInterval(quality_string, named_interval_number)

    return named_interval


# -*- encoding: utf-8 -*-
import copy
from abjad.tools.datastructuretools.TypedList import TypedList


class TempoInventory(TypedList):
    r'''An ordered list of tempo indications.

    ::

        >>> inventory = indicatortools.TempoInventory([
        ...     (Duration(1, 8), 72, 'Andante'),
        ...     (Duration(1, 8), 84, 'Allegro'),
        ...     ])

    ::

        >>> for tempo in inventory:
        ...     tempo
        ...
        Tempo(duration=Duration(1, 8), units_per_minute=72, textual_indication='Andante')
        Tempo(duration=Duration(1, 8), units_per_minute=84, textual_indication='Allegro')

    Tempo inventories implement list interface and are mutable.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### SPECIAL METHODS ###

    # TODO: put testable code in doctest
    def __illustrate__(self):
        r'''Illustrates tempo inventory.

        ::

            >>> show(inventory) # doctest: +SKIP

        Returns LilyPond file.
        '''
        from abjad.tools import durationtools
        from abjad.tools import lilypondfiletools
        from abjad.tools import indicatortools
        from abjad.tools import markuptools
        from abjad.tools import pitchtools
        from abjad.tools import scoretools
        from abjad.tools import spannertools
        from abjad.tools.topleveltools import attach
        from abjad.tools.topleveltools import iterate
        from abjad.tools.topleveltools import override
        from abjad.tools.topleveltools import new
        staff = scoretools.Staff()
        score = scoretools.Score([staff])
        time_signature = indicatortools.TimeSignature((2, 4))
        attach(time_signature, staff)
        # the zero note avoids a lilypond spacing problem:
        # score-initial tempo indications slip to the left
        zero_note = scoretools.Note("c'2")
        staff.append(zero_note)
        command = indicatortools.LilyPondCommand('break')
        attach(command, zero_note)
        for tempo in self.items:
            note = scoretools.Note("c'2")
            attach(tempo, note)
            staff.append(note)
            command = indicatortools.LilyPondCommand('break')
            attach(command, note)
        override(score).bar_line.transparent = True
        override(score).bar_number.stencil = False
        override(score).clef.stencil = False
        override(score).note_head.no_ledgers = True
        override(score).note_head.transparent = True
        override(score).staff_symbol.transparent = True
        override(score).stem.transparent = True
        override(score).time_signature.stencil = False
        lilypond_file = lilypondfiletools.make_basic_lilypond_file(score)
        lilypond_file.layout_block.indent = 0
        lilypond_file.layout_block.ragged_right = True
        lilypond_file.items.remove(lilypond_file['paper'])
        lilypond_file.header_block.tagline = markuptools.Markup('""')
        return lilypond_file

    ### PRIVATE PROPERTIES ###

    @property
    def _attribute_manifest(self):
        from abjad.tools import systemtools
        return systemtools.AttributeManifest()

    @staticmethod
    def _item_callable(expr):
        from abjad.tools import indicatortools
        if isinstance(expr, tuple):
            tempo = indicatortools.Tempo(*expr)
        elif isinstance(expr, indicatortools.Tempo):
            tempo = copy.copy(expr)
        else:
            raise TypeError(repr(expr))
        return tempo


# -*- coding: utf-8 -*-
import os
import shutil
import tempfile


def show(expr):
    r'''A replacement for Ajbad's show function for IPython Notebook.
    '''
    from abjad.tools import systemtools
    from abjad.tools import topleveltools
    from IPython.core.display import display_png
    assert '__illustrate__' in dir(expr)
    temporary_directory = tempfile.mkdtemp()
    temporary_file_path = os.path.join(
        temporary_directory,
        'output.png',
        )
    result = topleveltools.persist(expr).as_png(temporary_file_path)
    pngs = []
    for file_path in result[0]:
        command = 'convert {file_path} -trim {file_path}'.format(
            file_path=file_path,
            )
        systemtools.IOManager.spawn_subprocess(command)
        with open(file_path, 'rb') as file_pointer:
            file_contents = file_pointer.read()
            pngs.append(file_contents)
    shutil.rmtree(temporary_directory)
    for png in pngs:
        display_png(png, raw=True)


def load_ipython_extension(ipython):
    import abjad
    from abjad.tools import topleveltools
    abjad.show = show
    topleveltools.show = show
    ipython.push({'show': show})



# -*- encoding: utf-8 -*-
import abc
from abjad.tools.abctools.AbjadObject import AbjadObject


class BaseResidueClass(AbjadObject):
    r'''Abstract base class for ResidueClass and Sieve.
    '''

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### SPECIAL METHODS ###

    # TODO: implement __neg__() #

    def __and__(self, arg):
        r'''Logical AND of residue class and `arg`.

        Returns sieve.
        '''
        assert isinstance(arg, BaseResidueClass)
        return self._operate(arg, 'and')

    def __or__(self, arg):
        r'''Logical OR of residue class and `arg`.

        Returns sieve.
        '''
        assert isinstance(arg, BaseResidueClass)
        return self._operate(arg, 'or')

    def __xor__(self, arg):
        r'''Logical XOR of residue class and `arg`.

        Returns sieve.
        '''
        assert isinstance(arg, BaseResidueClass)
        return self._operate(arg, 'xor')

    ### PRIVATE METHODS ###

    def _operate(self, arg, op):
        from abjad.tools.sievetools.Sieve import Sieve
        if isinstance(self, Sieve) and self.logical_operator == op:
            argA = self.rcs
        else:
            argA = [self]
        if isinstance(arg, Sieve) and arg.logical_operator == op:
            argB = arg.rcs
        else:
            argB = [arg]
        return Sieve(argA + argB, op)

    @staticmethod
    def _process_min_max_attribute(*min_max):
        r'''Process minimum and maximum attributes.
        The function expects at least one and at most two attributes.
        Lone argument taken as range maximum.
        '''
        if len(min_max) == 0 or 2 < len(min_max):
            raise ValueError(min_max)
        elif len(min_max) == 1:
            minimum = 0
            maximum = min_max[0]
        else:
            minimum = min_max[0]
            maximum = min_max[1]
        if not (minimum < maximum and
            isinstance(minimum, int) and
            isinstance(maximum, int)):
            message = 'arguments must be integers and min < max.'
            raise ValueError(message)
        return minimum, maximum


# -*- encoding: utf-8 -*-
from __future__ import print_function
import docutils
import hashlib
import os
import pickle
import posixpath
import shutil
import sphinx
import subprocess
import tempfile
import traceback
from abjad import abjad_configuration
from abjad.tools import documentationtools
from abjad.tools import systemtools
try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO


class abjad_book_block(docutils.nodes.General, docutils.nodes.Element):
    pass


class abjad_literal_block(docutils.nodes.General, docutils.nodes.Element):
    pass


class AbjadBookDirective(sphinx.util.compat.Directive):
    has_content = True
    required_arguments = 0
    optional_arguments = 0
    final_argument_whitespace = True
    option_spec = {
        'errors-ok': docutils.parsers.rst.directives.flag,
        'hidden': docutils.parsers.rst.directives.flag,
        'strip-prompt': docutils.parsers.rst.directives.flag,
        }

    def run(self):
        self.assert_has_content()
        code = u'\n'.join(self.content)
        literal = abjad_literal_block(code, code)
        literal['errors-ok'] = 'errors-ok' in self.options
        literal['hidden'] = 'hidden' in self.options
        literal['strip-prompt'] = 'strip-prompt' in self.options
        #literal = docutils.nodes.literal_block(code, code)
        sphinx.util.nodes.set_source_info(self, literal)
        return [literal]


class ShellDirective(sphinx.util.compat.Directive):
    has_content = True
    required_arguments = 0
    optional_arguments = 0
    final_argument_whitespace = False
    option_spec = {}

    def run(self):
        self.assert_has_content()
        #original_directory = os.path.abspath(os.path.curdir)
        os.chdir(abjad_configuration.abjad_directory)
        result = []
        for line in self.content:
            prompt = '{}$ '.format(os.path.basename(os.path.abspath(os.path.curdir)))
            prompt += line
            result.append(prompt)
            proc = subprocess.Popen(
                line,
                #line.split(),
                shell=True,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                )
            out, err = proc.communicate()
            result.extend(out.splitlines())
            result.extend(err.splitlines())
        code = u'\n'.join(result)
        literal = docutils.nodes.literal_block(code, code)
        literal['language'] = 'console'
        sphinx.util.nodes.set_source_info(self, literal)
#        if literal.rawsource != literal.astext():
#            print 'RAW:\n' + literal.rawsource
#            print 'TXT:\n' + literal.asText()
#            literal.rawsource = literal.asText()
        return [literal]

#        literal = docutils.nodes.literal_block(code, code)
#        literal['language'] = 'bash'
#        sphinx.util.nodes.set_source_info(self, literal)
#        os.chdir(original_directory)
#        return [literal]


def on_builder_inited(app):
    tmp_directory = app.builder._abjad_book_tempdir = \
        os.path.abspath(tempfile.mkdtemp(dir=app.builder.outdir))
    if hasattr(app.builder, 'imgpath'):
        img_directory = os.path.join(app.builder.outdir, '_images')
        if not os.path.exists(img_directory):
            os.makedirs(img_directory)


def rewrite_literal_block_line(line):
    if line.strip().startswith(('f(', 'play(', 'print ', 'redo(', 'z(', 'systemtools.log(')):
        return '', False
    elif not line.startswith(('show(', 'topleveltools.graph(')):
        return line, False
    if line.startswith('show('):
        object_name = line[5:]
        kind = 'lilypond'
    elif line.startswith('topleveltools.graph('):
        object_name = line[len('topleveltools.graph('):]
        kind = 'graphviz'
    object_name = object_name.rpartition(')')[0]
    if ')' in object_name:
        object_name = object_name.rpartition(')')[0] + ')'
    elif ',' in object_name:
        object_name = object_name.rpartition(',')[0]
    object_name = object_name.strip()
    if kind in ('lilypond', 'graphviz'):
        return '__abjad_book__ = ({!r}, {})'.format(kind, object_name), True


def scan_doctree_for_literal_blocks(doctree):
    def is_valid_node(node):
        if isinstance(node,
            (docutils.nodes.literal_block, docutils.nodes.doctest_block)
            ):
            return True
        return False
    should_process = False
    literal_blocks = [x for x in doctree.traverse(is_valid_node)]
    literal_lines = []
    for literal_block in literal_blocks:
        lines = []
        for i, line in enumerate(literal_block[0].splitlines()):
            if line.startswith(('>>>', '... ')):
                rewritten_line, has_image = rewrite_literal_block_line(line[4:])
                should_process = should_process or has_image
                lines.append((i, rewritten_line))
        literal_lines.append(tuple(lines))
    return zip(literal_blocks, literal_lines), should_process


def scan_doctree_for_abjad_literal_blocks(doctree):
    abjad_literal_blocks = [x for x in doctree.traverse(abjad_literal_block)]
    should_process = 0 < len(abjad_literal_blocks)
    return abjad_literal_blocks, should_process


def process_literal_block_pairs(literal_block_pairs):
    environment = {
        '__builtins__': __builtins__,
        'print_function': print_function,
        }
    string_io = StringIO()
    with systemtools.RedirectedStreams(stdout=string_io):
        exec('from abjad import *\n', environment)
        for literal_block, all_lines in literal_block_pairs:
            original_lines = literal_block[0].splitlines()
            replacement_blocks = []
            lines_to_execute = []
            previous_line_number = 0
            for i, line in all_lines:
                lines_to_execute.append(line)
                if line.startswith('__abjad_book__ ='):
                    if '__abjad_book__' in environment:
                        del(environment['__abjad_book__'])
                    try:
                        exec('\n'.join(lines_to_execute), environment)
                    except Exception as e:
                        traceback.print_exc()
                    kind, obj = environment['__abjad_book__']
                    new_abjad_book_block = abjad_book_block()
                    new_abjad_book_block['kind'] = kind
                    if kind == 'lilypond':
                        lilypond_file = documentationtools.make_reference_manual_lilypond_file(obj)
                        new_abjad_book_block['code'] = format(lilypond_file)
                        new_abjad_book_block['raw_code'] = format(obj)
                    elif kind == 'graphviz':
                        graphviz_graph = documentationtools.make_reference_manual_graphviz_graph(obj)
                        new_abjad_book_block['code'] = graphviz_graph.graphviz_format
                    text = '\n'.join(original_lines[previous_line_number:i + 1])
                    new_literal_block = literal_block.deepcopy()
                    new_literal_block.rawsource = text
                    new_literal_block[0].rawsource = text
                    new_literal_block[0].text = text
                    replacement_blocks.extend([new_literal_block, new_abjad_book_block])
                    lines_to_execute = []
                    previous_line_number = i + 1
            if lines_to_execute:
                try:
                    exec('\n'.join(lines_to_execute), environment)
                except:
                    pass
                if replacement_blocks:
                    text = '\n'.join(original_lines[previous_line_number:])
                    new_literal_block = literal_block.deepcopy()
                    new_literal_block.rawsource = text
                    new_literal_block[0].rawsource = text
                    new_literal_block[0].text = text
                    replacement_blocks.append(new_literal_block)
            if replacement_blocks:
                literal_block.replace_self(replacement_blocks)


def process_abjad_literal_blocks(abjad_literal_blocks):
    # setup pipe
    pipe = documentationtools.Pipe()
    pipe.read_wait()
    pipe.write('from abjad import *\n')
    pipe.read_wait()

    for abjad_literal_block in abjad_literal_blocks:
        pass

    # cleanup pipe
    pipe.write('quit()\n')
    pipe.close()


def on_doctree_read(app, doctree):
    transform_path = app.config.abjad_book_transform_path
    docname = doctree['source'][:-4].partition(app.srcdir)[-1][1:]
    if not docname.startswith(transform_path):
        return
    result_a = scan_doctree_for_literal_blocks(doctree)
    result_b = scan_doctree_for_abjad_literal_blocks(doctree)
    should_process = result_a[1] or result_b[1]
    if not should_process:
        return
    literal_block_pairs = result_a[0]
    abjad_literal_blocks = result_b[0]
    process_literal_block_pairs(literal_block_pairs)
    #process_abjad_literal_blocks(abjad_literal_blocks)


def on_build_finished(app, exc):
    if os.path.exists(app.builder._abjad_book_tempdir):
        shutil.rmtree(app.builder._abjad_book_tempdir)


def render_graphviz_image(self, code, paths, file_format='png',
    is_latex=False, is_pickled=False, keep_original=False):
    assert file_format in ('png', 'pdf')
    primary_path = paths['primary_absolute_path']
    secondary_path = paths['secondary_absolute_path']
    tmp_path = os.path.join(self.builder._abjad_book_tempdir,
        os.path.basename(os.path.splitext(primary_path)[0])) + '.dot'
    # if we pickled a documentationtools.GraphvizGraph instance
    # in order to support conditional reformatting for LaTeX vs HTML...

    if is_pickled:
        graph = pickle.loads(code)
        alt = graph.graphviz_format
        if is_latex:
            graph.attributes['fontsize'] = 10
            graph.attributes['ranksep'] = 0.25
            graph.attributes['ratio'] = 'compress'
            graph.attributes['size'] = 6.5
            graph.node_attributes['fontsize'] = 8
            graph.node_attributes['margin'] = (0.06, 0.06)
        code = graph.unflattened_graphviz_format
    else:
        alt = code
    with open(tmp_path, 'w') as f:
        f.write(code)
    commands = []
    if file_format == 'pdf':
        commands.append('dot -Tpdf -o {} {}'.format(primary_path, tmp_path))
        commands.append('pdfcrop {} {}'.format(primary_path, primary_path))
    elif file_format == 'png':
        commands.append('dot -Tpdf -o {} {}'.format(secondary_path, tmp_path))
        commands.append('dot -Tpng -o {} {}'.format(primary_path, tmp_path))
        commands.append('convert -debug Exception -trim -resize 50%% -resize 800x9999">" {} {}'.format(
            primary_path, primary_path))
    for command in commands:
        subprocess.call(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        #subprocess.call(command, shell=True)
    if not keep_original and os.path.exists(secondary_path):
        os.remove(secondary_path)
    return alt


def render_lilypond_image(self, code, paths, file_format='png', keep_original=False):
    assert file_format in ('png', 'pdf')
    absolute_path = paths['primary_absolute_path']
    # LilyPond insists on appending an extension, even if you already did it yourself.
    abs_path = os.path.splitext(absolute_path)[0]
    tmp_path = os.path.join(self.builder._abjad_book_tempdir,
        os.path.basename(os.path.splitext(absolute_path)[0])) + '.ly'
    with open(tmp_path, 'w') as f:
        f.write(code)
    commands = []
    if file_format == 'png':
        commands.append('lilypond --png -dresolution=300 -o {} {}'.format(abs_path, tmp_path))
        commands.append('convert -trim -resample 50%% {} {}'.format(absolute_path, absolute_path))
#        commands.append('lilypond --png -dresolution=600 -o {} {}'.format(abs_path, tmp_path))
#        commands.append('convert -units PixelsPerInch {} -trim -resample 150 -density 72 {}'.format(
#            absolute_path,
#            absolute_path,
#            ))
    elif file_format == 'pdf':
        commands.append('lilypond -o {} {}'.format(abs_path, tmp_path))
        commands.append('pdfcrop {} {}'.format(absolute_path, absolute_path))
    for command in commands:
        subprocess.call(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)

def render_abjad_book_node(self, node, file_format='png', linked=False):
    code = node['code']
    kind = node['kind']
    keep_original = node.get('keep_original', False)
    is_pickled = node.get('is_pickled', False)
    suffix = 'original'
    hashkey = code.encode('utf-8') + kind
    hexdigest = hashlib.sha1(hashkey).hexdigest()
    # primary in the target format, but secondary always in full-quality PDF
    primary_file_name = '{}-{}.{}'.format(
        kind, hexdigest, file_format)
    secondary_file_name = '{}-{}-{}.pdf'.format(
        kind, hexdigest, suffix)
    paths = {}
    if hasattr(self.builder, 'imgpath'): # HTML
        is_latex = False
        img_directory = os.path.join(self.builder.outdir, '_images')
        if not os.path.exists(img_directory):
            os.makedirs(img_directory)
        paths['primary_relative_path'] = posixpath.join(
            self.builder.imgpath, primary_file_name)
        paths['primary_absolute_path'] = os.path.join(
            self.builder.outdir, '_images', primary_file_name)
        paths['secondary_relative_path'] = posixpath.join(
            self.builder.imgpath, secondary_file_name)
        paths['secondary_absolute_path'] = os.path.join(
            self.builder.outdir, '_images', secondary_file_name)
    else: # LaTeX
        is_latex = True
        paths['primary_relative_path'] = primary_file_name
        paths['primary_absolute_path'] = os.path.join(
            self.builder.outdir, primary_file_name)
        paths['secondary_relative_path'] = secondary_file_name
        paths['secondary_absolute_path'] = os.path.join(
            self.builder.outdir, secondary_file_name)
    # TODO: The `alt` handling here is completely hackish, and must be
    #       refactored with the next pass on abjadbooktools.
    alt = node.get('raw_code', '')
    if os.path.isfile(paths['primary_absolute_path']):
        return paths, alt
    if kind == 'lilypond':
        render_lilypond_image(self, code, paths, file_format,
            keep_original=keep_original)
    elif kind == 'graphviz':
        alt = render_graphviz_image(self, code, paths, file_format,
            is_latex=is_latex,
            is_pickled=is_pickled,
            keep_original=keep_original,
            )
    else:
        raise Exception('Unknown node kind, {!r}'.format(kind))
    return paths, alt


def visit_abjad_book_html(self, node):
    result = render_abjad_book_node(self, node, file_format='png')
    paths, alt = result
    alt = self.encode(alt).strip()
    self.body.append('<div class="abjad-book">\n')
    self.body.append('<img src="{}" alt="Click to view source." title="Click to view source."/>\n'.format(
        paths['primary_relative_path']))
    self.body.append('<pre>\n{}\n</pre>\n'.format(alt))
    self.body.append('</div>\n')
    raise docutils.nodes.SkipNode


def visit_abjad_book_latex(self, node):
    paths, alt = render_abjad_book_node(
        self, node, file_format='pdf',
        )
    self.body.append('\n\\includegraphics{' + paths['primary_relative_path'] + '}\n')
    raise docutils.nodes.SkipNode


def setup(app):
    app.add_directive('abjad', AbjadBookDirective)
    app.add_directive('shell', ShellDirective)
    app.add_node(abjad_book_block,
        html=(visit_abjad_book_html, None),
        latex=(visit_abjad_book_latex, None),
        # we can add more rendering options later
        #texinfo=(visit_abjad_book_texinfo, None),
        #text=(visit_abjad_book_text, None),
        #man=(visit_abjad_book_man, None)
        )
    app.connect('builder-inited', on_builder_inited)
    app.connect('doctree-read', on_doctree_read)
    app.connect('build-finished', on_build_finished)
    app.add_config_value('abjad_book_transform_path', 'api/tools/', 'env')



# -*- encoding: utf-8 -*-


def get_numbered_pitch_class_from_pitch_carrier(pitch_carrier):
    '''Get numbered pitch-class from `pitch_carrier`:

    ::

        >>> note = Note("cs'4")
        >>> pitchtools.get_numbered_pitch_class_from_pitch_carrier(note)
        NumberedPitchClass(1)

    Raise missing pitch error on empty chords.

    Raise extra pitch error on many-note chords.

    Returns numbered pitch-class.
    '''
    from abjad.tools import pitchtools

    pitch = pitchtools.get_named_pitch_from_pitch_carrier(pitch_carrier)
    pitch_class = pitchtools.NumberedPitchClass(pitch)

    return pitch_class


# -*- encoding: utf-8 -*-
from abjad import *


def test_datastructuretools_CyclicTree___init___01():
    r'''Initializes cyclic tree from other cyclic tree.
    '''

    tree_1 = datastructuretools.CyclicPayloadTree([[4, 5], [6, 7]])
    tree_2 = datastructuretools.CyclicPayloadTree(tree_1)

    assert tree_1 is not tree_2
    assert tree_1 == tree_2


# -*- encoding: utf-8 -*-
from abjad.tools import indicatortools
from abjad.tools import markuptools
from abjad.tools import pitchtools
from abjad.tools.instrumenttools.Instrument import Instrument


class AltoTrombone(Instrument):
    r'''An alto trombone.

    ::

        >>> staff = Staff("c4 d4 e4 fs4")
        >>> clef = Clef(name='bass')
        >>> attach(clef, staff)
        >>> alto_trombone = instrumenttools.AltoTrombone()
        >>> attach(alto_trombone, staff)
        >>> show(staff) # doctest: +SKIP

    ..  doctest::

        >>> print(format(staff))
        \new Staff {
            \clef "bass"
            \set Staff.instrumentName = \markup { Alto trombone }
            \set Staff.shortInstrumentName = \markup { Alt. trb. }
            c4
            d4
            e4
            fs4
        }

    '''

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        instrument_name='alto trombone',
        short_instrument_name='alt. trb.',
        instrument_name_markup=None,
        short_instrument_name_markup=None,
        allowable_clefs=('bass', 'tenor'),
        pitch_range='[A2, Bb5]',
        sounding_pitch_of_written_middle_c=None,
        ):
        Instrument.__init__(
            self,
            instrument_name=instrument_name,
            short_instrument_name=short_instrument_name,
            instrument_name_markup=instrument_name_markup,
            short_instrument_name_markup=short_instrument_name_markup,
            allowable_clefs=allowable_clefs,
            pitch_range=pitch_range,
            sounding_pitch_of_written_middle_c=\
                sounding_pitch_of_written_middle_c,
            )
        self._performer_names.extend([
            'brass player',
            'trombonist',
            ])
        self._starting_clefs = indicatortools.ClefInventory(['bass'])

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r'''Formats alto trombone.

        Set `format_specification` to `''` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        ..  container:: example

            ::

                >>> alto_trombone = instrumenttools.AltoTrombone()
                >>> print(format(alto_trombone))
                instrumenttools.AltoTrombone(
                    instrument_name='alto trombone',
                    short_instrument_name='alt. trb.',
                    instrument_name_markup=markuptools.Markup(
                        contents=('Alto trombone',),
                        ),
                    short_instrument_name_markup=markuptools.Markup(
                        contents=('Alt. trb.',),
                        ),
                    allowable_clefs=indicatortools.ClefInventory(
                        [
                            indicatortools.Clef(
                                name='bass',
                                ),
                            indicatortools.Clef(
                                name='tenor',
                                ),
                            ]
                        ),
                    pitch_range=pitchtools.PitchRange(
                        range_string='[A2, Bb5]',
                        ),
                    sounding_pitch_of_written_middle_c=pitchtools.NamedPitch("c'"),
                    )

        Returns string.
        '''
        superclass = super(AltoTrombone, self)
        return superclass.__format__(format_specification=format_specification)

    ### PUBLIC PROPERTIES ###

    @property
    def allowable_clefs(self):
        r'''Gets alto trombone's allowable clefs.

        ..  container:: example

            ::

                >>> alto_trombone.allowable_clefs
                ClefInventory([Clef(name='bass'), Clef(name='tenor')])

            ::

                >>> show(alto_trombone.allowable_clefs) # doctest: +SKIP

        Returns clef inventory.
        '''
        return Instrument.allowable_clefs.fget(self)

    @property
    def instrument_name(self):
        r'''Gets alto trombone's name.

        ..  container:: example

            ::

                >>> alto_trombone.instrument_name
                'alto trombone'

        Returns string.
        '''
        return Instrument.instrument_name.fget(self)

    @property
    def instrument_name_markup(self):
        r'''Gets alto trombone's instrument name markup.

        ..  container:: example

            ::

                >>> alto_trombone.instrument_name_markup
                Markup(contents=('Alto trombone',))

            ::

                >>> show(alto_trombone.instrument_name_markup) # doctest: +SKIP

        Returns markup.
        '''
        return Instrument.instrument_name_markup.fget(self)

    @property
    def pitch_range(self):
        r'''Gets alto trombone's range.

        ..  container:: example

            ::

                >>> alto_trombone.pitch_range
                PitchRange(range_string='[A2, Bb5]')

            ::

                >>> show(alto_trombone.pitch_range) # doctest: +SKIP

        Returns pitch range.
        '''
        return Instrument.pitch_range.fget(self)

    @property
    def short_instrument_name(self):
        r'''Gets alto trombone's short instrument name.

        ..  container:: example

            ::

                >>> alto_trombone.short_instrument_name
                'alt. trb.'

        Returns string.
        '''
        return Instrument.short_instrument_name.fget(self)

    @property
    def short_instrument_name_markup(self):
        r'''Gets alto trombone's short instrument name markup.

        ..  container:: example

            ::

                >>> alto_trombone.short_instrument_name_markup
                Markup(contents=('Alt. trb.',))

            ::

                >>> show(alto_trombone.short_instrument_name_markup) # doctest: +SKIP

        Returns markup.
        '''
        return Instrument.short_instrument_name_markup.fget(self)

    @property
    def sounding_pitch_of_written_middle_c(self):
        r'''Gets sounding pitch of alto trombone's written middle C.

        ..  container:: example

            ::

                >>> alto_trombone.sounding_pitch_of_written_middle_c
                NamedPitch("c'")

            ::

                >>> show(alto_trombone.sounding_pitch_of_written_middle_c) # doctest: +SKIP

        Returns named pitch.
        '''
        return Instrument.sounding_pitch_of_written_middle_c.fget(self)


# -*- encoding: utf-8 -*-
import os
from abjad.tools import documentationtools
from abjad.tools import systemtools
from abjad.tools.developerscripttools.DeveloperScript import DeveloperScript
from abjad.tools.developerscripttools.ReplaceInFilesScript \
    import ReplaceInFilesScript


class RenameModulesScript(DeveloperScript):
    r'''Renames classes and functions.

    Handle renaming the module and package, as well as any tests,
    documentation or mentions of the class throughout the Abjad codebase:

    ..  shell::

        ajv rename --help

    '''

    ### PUBLIC PROPERTIES ###

    @property
    def alias(self):
        r'''Alias of script.

        Returns ``'rename'``.
        '''
        return 'rename'

    @property
    def long_description(self):
        r'''Long description of script.

        Returns string or none.
        '''
        return None

    @property
    def scripting_group(self):
        r'''Scripting group of script.

        Returns none.
        '''
        return None

    @property
    def short_description(self):
        r'''Short description of script.

        Returns string.
        '''
        return 'Rename public modules.'

    @property
    def version(self):
        r'''Version of script.

        Returns float.
        '''
        return 1.0

    ### PRIVATE METHODS ###

    def _codebase_name_to_codebase_docs_path(self, codebase):
        from abjad import abjad_configuration
        if codebase == 'mainline':
            return os.path.join(
                abjad_configuration.abjad_directory,
                'docs',
                'source',
                'api',
                'tools',
                )
        elif codebase == 'experimental':
            return os.path.join(
                abjad_configuration.abjad_experimental_directory,
                'docs',
                'source',
                'tools',
                )
        message = 'bad codebase name: {!r}.'
        message = message.format(codebase)
        raise Exception(message)

    def _codebase_name_to_codebase_tools_path(self, codebase):
        from abjad import abjad_configuration
        if codebase == 'mainline':
            return os.path.join(
                abjad_configuration.abjad_directory, 'tools')
        elif codebase == 'experimental':
            return os.path.join(
                abjad_configuration.abjad_experimental_directory, 'tools')
        message = 'bad codebase name: {!r}.'
        message = message.format(codebase)
        raise Exception(message)

    def _confirm_name_changes(self,
        old_codebase,
        old_tools_package_name,
        old_module_name,
        new_codebase,
        new_tools_package_name,
        new_module_name,
        ):
        max_codebase = max(len(old_codebase), len(new_codebase))
        old_codebase = old_codebase.ljust(max_codebase)
        new_codebase = new_codebase.ljust(max_codebase)
        print('')
        print('Is ...')
        print('')
        print('    [{}] {}.{}()'.format(
            old_codebase, old_tools_package_name, old_module_name))
        print('    ===>')
        print('    [{}] {}.{}()'.format(
            new_codebase, new_tools_package_name, new_module_name))
        print('')
        string = raw_input('... correct [yes, no, abort]? ').lower()
        print('')
        if string in ('y', 'yes'):
            return True
        elif string in ('a', 'abort', 'q', 'quit'):
            raise SystemExit
        elif string in ('n', 'no'):
            return False

    def _get_object_names(self, kind, codebase, tools_package_name):
        assert kind in ('class', 'function')
        tools_path = self._codebase_name_to_codebase_tools_path(codebase)
        path = os.path.join(tools_path, tools_package_name)
        if kind == 'class':
            crawler = documentationtools.ClassCrawler(
                path, include_private_objects=True)
        elif kind == 'function':
            crawler = documentationtools.FunctionCrawler(
                path, include_private_objects=True)
        objs = crawler()
        return tuple(sorted([x.__name__ for x in objs]))

    def _get_tools_package_names(self, codebase):
        tools_path = self._codebase_name_to_codebase_tools_path(codebase)
        names = []
        for x in os.listdir(tools_path):
            if os.path.isdir(os.path.join(tools_path, x)):
                if not x.startswith(('_', '.')):
                    names.append(x)
        return tuple(sorted(names))

    def _parse_tools_package_path(self, path):
        from abjad import abjad_configuration
        if '.' not in path:
            raise SystemExit
        tools_package_name, module_name = path.split('.')
        mainline_tools_directory = os.path.join(
            abjad_configuration.abjad_directory,
            'tools',
            )
        for directory_name in os.listdir(mainline_tools_directory):
            directory = os.path.join(
                mainline_tools_directory, directory_name)
            if not os.path.isdir(directory):
                continue
            elif directory_name != tools_package_name:
                continue
            return 'mainline', tools_package_name, module_name
        experimental_tools_directory = os.path.join(
            abjad_configuration.abjad_experimental_directory,
            'tools',
            )
        for directory_name in os.listdir(mainline_tools_directory):
            directory = os.path.join(
                experimental_tools_directory, directory_name)
            if not os.path.isdir(directory):
                continue
            elif directory_name != tools_package_name:
                continue
            return 'experimental', tools_package_name, module_name
        raise SystemExit

    def _rename_old_api_page(self,
        old_codebase,
        old_tools_package_name,
        old_module_name,
        new_codebase,
        new_tools_package_name,
        new_module_name,
        ):
        print('Renaming old API page ...')
        old_docs_path = self._codebase_name_to_codebase_docs_path(old_codebase)
        new_docs_path = self._codebase_name_to_codebase_docs_path(new_codebase)
        old_rst_file_name = old_module_name + '.rst'
        new_rst_file_name = new_module_name + '.rst'
        old_api_path = os.path.join(
            old_docs_path, old_tools_package_name, old_rst_file_name)
        new_api_path = os.path.join(
            new_docs_path, new_tools_package_name, new_rst_file_name)
        command = 'mv {} {}'.format(
            old_api_path, new_api_path)
        systemtools.IOManager.spawn_subprocess(command)
        print('')

    def _rename_old_module(self,
        old_codebase,
        old_tools_package_name,
        old_module_name,
        new_codebase,
        new_tools_package_name,
        new_module_name,
        ):
        print('Renaming old module ...')
        old_tools_path = self._codebase_name_to_codebase_tools_path(
            old_codebase)
        new_tools_path = self._codebase_name_to_codebase_tools_path(
            new_codebase)
        old_module = old_module_name + '.py'
        old_path = os.path.join(
            old_tools_path, old_tools_package_name, old_module)
        new_module = new_module_name + '.py'
        new_path = os.path.join(
            new_tools_path, new_tools_package_name, new_module)
        command = 'git mv -f {} {}'.format(
            old_path, new_path)
        systemtools.IOManager.spawn_subprocess(command)
        print('')

    def _rename_old_test_files(self,
        old_codebase,
        old_tools_package_name,
        old_module_name,
        new_codebase,
        new_tools_package_name,
        new_module_name,
        ):
        print('Renaming old test file(s) ...')
        old_tools_path = self._codebase_name_to_codebase_tools_path(
            old_codebase)
        old_test_path = os.path.join(
            old_tools_path, old_tools_package_name, 'test')
        if not os.path.exists(old_test_path):
            return
        new_tools_path = self._codebase_name_to_codebase_tools_path(
            new_codebase)
        new_test_path = os.path.join(
            new_tools_path, new_tools_package_name, 'test')
        old_test_file_prefix = 'test_{}_{}'.format(
            old_tools_package_name, old_module_name)
        old_test_file_names = [x for x in os.listdir(old_test_path)
            if x.startswith(old_test_file_prefix) and x.endswith('.py')]
        for old_test_file_name in old_test_file_names:
            old_test_file_path = os.path.join(
                old_test_path, old_test_file_name)
            old_test_file_suffix = old_test_file_name[
                len(old_test_file_prefix):]
            new_test_file_name = 'test_{}_{}{}'.format(
                new_tools_package_name, new_module_name, old_test_file_suffix)
            new_test_file_path = os.path.join(
                new_test_path, new_test_file_name)
            command = 'git mv -f {} {}'.format(
                old_test_file_path, new_test_file_path)
            systemtools.IOManager.spawn_subprocess(command)
        print('')

    def _update_codebase(self,
        old_codebase,
        old_tools_package_name,
        old_module_name,
        new_codebase,
        new_tools_package_name,
        new_module_name,
        ):
        from abjad import abjad_configuration
        without_dirs = ['--without-dirs', 'build', '--without-dirs', '_build']
        directory = abjad_configuration.abjad_root_directory
        print('Updating codebase ...')
        print('')
        old_text = '{}.{}'.format(old_tools_package_name, old_module_name)
        new_text = '{}.{}'.format(new_tools_package_name, new_module_name)
        command = [
            directory,
            old_text,
            new_text,
            '--force',
            '--whole-words-only',
            #'--verbose',
            ]
        command.extend(without_dirs)
        ReplaceInFilesScript()(command)
        print('')
        old_text = 'test_{}_{}_'.format(
            old_tools_package_name, old_module_name)
        new_text = 'test_{}_{}_'.format(
            new_tools_package_name, new_module_name)
        command = [directory, old_text, new_text, '--force', '--verbose']
        command.extend(without_dirs)
        ReplaceInFilesScript()(command)
        print('')
        old_text = old_module_name
        new_text = new_module_name
        command = [
            directory,
            old_text,
            new_text,
            '--force',
            '--whole-words-only',
            #'--verbose',
            ]
        command.extend(without_dirs)
        ReplaceInFilesScript()(command)
        print('')

    ### PUBLIC METHODS ###

    def process_args(self, args):
        r'''Processes `args`.

        Returns none.
        '''
        systemtools.IOManager.clear_terminal()
        # Handle source path:
        old_codebase, old_tools_package_name, old_module_name = \
            self._parse_tools_package_path(args.source)
        old_codebase_tools_path = self._codebase_name_to_codebase_tools_path(
            old_codebase)
        old_module_path = os.path.join(
            old_codebase_tools_path,
            old_tools_package_name,
            old_module_name + '.py',
            )
        if not os.path.exists(old_module_path):
            message = 'source does not exist: {}'
            message = message.format(old_module_path)
            raise SystemExit(message)
        # Handle destination path:
        new_codebase, new_tools_package_name, new_module_name = \
            self._parse_tools_package_path(args.destination)
        new_codebase_tools_path = self._codebase_name_to_codebase_tools_path(
            new_codebase)
        new_module_path = os.path.join(
            new_codebase_tools_path,
            new_tools_package_name,
            new_module_name + '.py',
            )
        if os.path.exists(new_module_path):
            message = 'destination already exists: {}'
            message = message.format(old_module_path)
            raise SystemExit(message)
        # Process changes:
        new_args = (
            old_codebase, old_tools_package_name, old_module_name,
            new_codebase, new_tools_package_name, new_module_name,
            )
        if not self._confirm_name_changes(*new_args):
            raise SystemExit
        self._rename_old_test_files(*new_args)
        self._rename_old_api_page(*new_args)
        self._rename_old_module(*new_args)
        self._update_codebase(*new_args)
        raise SystemExit

    def setup_argument_parser(self, parser):
        r'''Sets up argument `parser`.

        Returns none.
        '''
        parser.add_argument(
            'source',
            help='toolspackage path of source module',
            )
        parser.add_argument(
            'destination',
            help='toolspackage path of destination module',
            )


# -*- encoding: utf-8 -*-
from abjad.tools import indicatortools
from abjad.tools import scoretools
from abjad.tools.topleveltools import iterate


def notes_and_chords_are_on_expected_clefs(
    expr,
    percussion_clef_is_allowed=True,
    ):
    r'''Is true when notes and chords in `expr` are on expected clefs.

    ::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> clef = Clef(name='treble')
        >>> attach(clef, staff)
        >>> violin = instrumenttools.Violin()
        >>> attach(violin, staff)

    ::

        >>> instrumenttools.notes_and_chords_are_on_expected_clefs(staff)
        True

    Otherwise false:

    ::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> clef = Clef(name='alto')
        >>> attach(clef, staff)
        >>> violin = instrumenttools.Violin()
        >>> attach(violin, staff)

    ::

        >>> instrumenttools.notes_and_chords_are_on_expected_clefs(staff)
        False

    Allows percussion clef when `percussion_clef_is_allowed` is true:

    ::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> clef = Clef(name='percussion')
        >>> attach(clef, staff)
        >>> violin = instrumenttools.Violin()
        >>> attach(violin, staff)

    ..  doctest::

        >>> print(format(staff))
        \new Staff {
            \clef "percussion"
            \set Staff.instrumentName = \markup { Violin }
            \set Staff.shortInstrumentName = \markup { Vn. }
            c'8
            d'8
            e'8
            f'8
        }

    ::

        >>> instrumenttools.notes_and_chords_are_on_expected_clefs(
        ...     staff, percussion_clef_is_allowed=True)
        True

    Disallows percussion clef when `percussion_clef_is_allowed` is false:

    ::

        >>> instrumenttools.notes_and_chords_are_on_expected_clefs(
        ...     staff, percussion_clef_is_allowed=False)
        False

    Returns boolean.
    '''
    from abjad.tools import instrumenttools
    for note_or_chord in iterate(expr).by_class(
        (scoretools.Note, scoretools.Chord)):
        instrument = note_or_chord._get_effective(
            instrumenttools.Instrument)
        if not instrument:
            return False
        clef = note_or_chord._get_effective(indicatortools.Clef)
        if not clef:
            return False
        if clef == indicatortools.Clef(name='percussion'):
            if percussion_clef_is_allowed:
                return True
            else:
                return False
        if clef not in instrument.allowable_clefs:
            return False
    else:
        return True



# -*- encoding: utf-8 -*-
from abjad import *


def test_sequencetools_partition_sequence_by_ratio_of_lengths_01():

    parts = sequencetools.partition_sequence_by_ratio_of_lengths(
        list(range(10)),
        [1, 1, 2],
        )
    assert parts == [[0, 1, 2], [3, 4], [5, 6, 7, 8, 9]]



# -*- encoding: utf-8 -*-


def make_blank_line_markup():
    r'''Make blank line markup:

    ::

        >>> markup = markuptools.make_blank_line_markup()

    ::

        >>> markup
        Markup(contents=(MarkupCommand('fill-line', [' ']),))

    ..  doctest::

        >>> print(format(markup))
        \markup { \fill-line { " " } }

    Returns markup.
    '''
    from abjad.tools import markuptools

    return markuptools.Markup(r'\fill-line { " " }')



# -*- encoding: utf-8 -*-
import argparse
import importlib
import os
from abjad.tools import documentationtools
from abjad.tools.developerscripttools.DirectoryScript import DirectoryScript


class CountLinewidthsScript(DirectoryScript):
    r'''Counts linewidths of modules in a path.

    ..  shell::

        ajv count linewidths --help

    '''

    ### PUBLIC PROPERTIES ###

    @property
    def alias(self):
        r'''Alias of script.

        Returns ``'linewidths'``.
        '''
        return 'linewidths'

    @property
    def long_description(self):
        r'''Long description of script.

        Returns string or none.
        '''
        return None

    @property
    def scripting_group(self):
        r'''Scripting group of script.

        Returns ``'count'``.
        '''
        return 'count'

    @property
    def short_description(self):
        r'''Short description of script.

        Returns string.
        '''
        return 'Count maximum line-width of all modules in PATH.'

    @property
    def version(self):
        r'''Version of script.

        Returns float.
        '''
        return 1.0

    ### PUBLIC METHODS ###

    def process_args(self, args):
        r'''Processes `args`.

        Returns none.
        '''

        #print args

        modules = {}

        if args.mode == 'docstrings':
            print('DOCS:\tMODULE:')
            for obj in documentationtools.FunctionCrawler(args.path)():
                docstring = obj.__doc__
                if not docstring:
                    modules[obj.__module__] = 0
                else:
                    lines = docstring.split('\n')
                    modules[obj.__module__] = \
                        max([len(line) for line in lines])
            for obj in documentationtools.ClassCrawler(args.path)():
                docstring = obj.__doc__
                width = 0
                if docstring:
                    lines = docstring.split('\n')
                    width = max([len(line) for line in lines])
                for attr_name in dir(obj):
                    attr = getattr(obj, attr_name)
                    attr_docstring = getattr(attr, '__doc__', None)
                    if attr_docstring:
                        lines = attr_docstring.split('\n')
                        attr_width = max([len(line) for line in lines])
                        if width < attr_width:
                            width = attr_width
                modules[obj.__module__] = width

        elif args.mode == 'code':
            print('CODE:\tMODULE:')
            for obj in documentationtools.FunctionCrawler(args.path)():
                module_path = obj.__module__
                module_obj = importlib.import_module(module_path)
                module_file_name = module_obj.__file__
                if module_file_name.endswith('.pyc'):
                    module_file_name = module_file_name[:-1]
                with open(module_file_name, 'r') as f:
                    lines = f.read().split('\n')
                    modules[obj.__module__] = \
                        max([len(line) for line in lines])
            for obj in documentationtools.ClassCrawler(args.path)():
                module_path = obj.__module__
                module_obj = importlib.import_module(module_path)
                module_file_name = module_obj.__file__
                if module_file_name.endswith('.pyc'):
                    module_file_name = module_file_name[:-1]
                with open(module_file_name, 'r') as f:
                    lines = f.read().split('\n')
                    modules[obj.__module__] = \
                        max([len(line) for line in lines])

        if args.order_by == 'm':
            order_by = lambda x: (x[0], x[1])
        else:
            order_by = lambda x: (x[1], x[0])

        if args.sort == 'descending':
            modules = sorted(list(modules.items()), key=order_by, reverse=True)
        else:
            modules = sorted(list(modules.items()), key=order_by)

        if args.greater_than is not None and 0 < args.greater_than:
            modules = [pair for pair in modules if args.greater_than < pair[1]]
        elif args.less_than is not None and 0 < args.less_than:
            modules = [pair for pair in modules if pair[1] < args.less_than]
        elif args.equal_to is not None and 0 <= args.equal_to:
            modules = [pair for pair in modules if pair[1] == args.equal_to]

        if args.limit is not None and 0 < args.limit:
            modules = modules[-args.limit:]

        for pair in modules:
            print('{}\t{}'.format(pair[1], pair[0]))

    def setup_argument_parser(self, parser):
        r'''Sets up argument `parser`.

        Returns none.
        '''
        parser.add_argument('path',
            type=self._validate_path,
            help='directory tree to be recursed over'
            )
        parser.add_argument('-l', '--limit',
            help='limit output to last N items',
            metavar='N',
            type=int,
            )
        parser.add_argument('-o', '--order-by',
            choices='wm',
            help='order by line width [w] or module name [m]',
            metavar='w|m',
            type=str,
            )
        mode_group = parser.add_mutually_exclusive_group()
        mode_group.add_argument('-C', '--code',
            action='store_const',
            const='code',
            dest='mode',
            help='count linewidths of all code in module',
            )
        mode_group.add_argument('-D', '--docstrings',
            action='store_const',
            const='docstrings',
            dest='mode',
            help='count linewidths of all docstrings in module',
            )
        sort_group = parser.add_mutually_exclusive_group()
        sort_group.add_argument('-a', '--ascending',
            action='store_const',
            const='ascending',
            dest='sort',
            help='sort results ascending',
            )
        sort_group.add_argument('-d', '--descending',
            action='store_const',
            const='descending',
            dest='sort',
            help='sort results descending',
            )
        thresh_group = parser.add_mutually_exclusive_group()
        thresh_group.add_argument('-gt', '--greater-than',
            help='line widths greater than N',
            metavar='N',
            type=int,
            )
        thresh_group.add_argument('-lt', '--less-than',
            help='line widths less than N',
            metavar='N',
            type=int,
            )
        thresh_group.add_argument('-eq', '--equal-to',
            help='line widths equal to N',
            metavar='N',
            type=int,
            )
        parser.set_defaults(mode='docstrings', order_by='w', sort='ascending')


# -*- encoding: utf-8 -*-
import types


# TODO: figure out how to handle functions with **kwargs
def requires(*tests):
    r'''Function decorator to require input parameter `tests`.

    ..  container:: example

        ::

            >>> @systemtools.requires(
            ...     mathtools.is_nonnegative_integer, string) # doctest: +SKIP
            >>> def multiply_string(n, string):
            ...     return n * string
            ...

        ::

            >>> multiply_string(2, 'bar') # doctest: +SKIP
            'barbar'

        ::

            >>> multiply_string(2.5, 'bar') # doctest: +SKIP
            ...
            AssertionError: is_nonnegative_integer(2.5) does not return true.

    Decorator target is available like this:

    ::

        >>> multiply_string.func_closure[1].cell_contents # doctest: +SKIP
        <function multiply_string at 0x104e512a8>

    Decorator tests are available like this:

    ::

        >>> multiply_string.func_closure[0].cell_contents # doctest: +SKIP
        (<function is_nonnegative_integer at 0x104725d70>, <type 'str'>)

    Returns decorated function in the form of function wrapper.
    '''
    def deco(func):
        '''
        Decorator function to be returned from requires().
        Returns a function wrapper that validates arguments.
        '''
        def wrapper (*args):
            '''
            Function wrapper that validates arguments.
            '''
            assert len(args) == len(tests), 'wrong number of arguments.'
            for arg, test in zip(args, tests):
                if isinstance(test, tuple):
                    tuple_repr = [x.__name__ for x in test]
                    tuple_repr = ', '.join(tuple_repr)
                    tuple_repr = '({})'.format(tuple_repr)
                    error_message = 'isinstance({!r}, {}) does not return true.'
                    error_message = error_message.format(arg, tuple_repr)
                    assert isinstance(arg, test), error_message
                elif isinstance(test, type):
                    error_message = 'isinstance({!r}, {}) does not return true.'
                    error_message = error_message.format(arg, test.__name__)
                    assert isinstance(arg, test), error_message
                else:
                    error_message = '{}({!r}) does not return true.'
                    error_message = error_message.format(test.__name__, arg)
                    assert test(arg), error_message
            return func(*args)
        # point Sphinx to correct docstring
        wrapper.__doc__ = func.__doc__
        return wrapper
    return deco



# -*- encoding: utf-8 -*-


def get_shared_numeric_sign(sequence):
    r'''Gets shared numeric sign of elements in `sequence`.

    Returns ``1`` when all `sequence` elements are positive:

    ::

        >>> mathtools.get_shared_numeric_sign([1, 2, 3])
        1

    Returns ``-1`` when all `sequence` elements are negative:

    ::

        >>> mathtools.get_shared_numeric_sign([-1, -2, -3])
        -1

    Returns ``0`` on empty `sequence`:

    ::

        >>> mathtools.get_shared_numeric_sign([])
        0

    Otherwise returns none:

    ::

        >>> mathtools.get_shared_numeric_sign([1, 2, -3]) is None
        True

    Returns ``1``, ``-1``, ``0`` or none.
    '''

    if len(sequence) == 0:
        return 0
    elif all(0 < x for x in sequence):
        return 1
    elif all(x < 0 for x in sequence):
        return -1
    else:
        return None


# -*- encoding: utf-8 -*-
from abjad.tools import mathtools
from abjad.tools import sequencetools
from abjad.tools.abctools import AbjadValueObject


class Talea(AbjadValueObject):
    '''Talea.

    ..  container:: example

        ::

            >>> talea = rhythmmakertools.Talea(
            ...    counts=(2, 1, 3, 2, 4, 1, 1),
            ...    denominator=16,
            ...    )

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_counts',
        '_denominator',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        counts=(1,),
        denominator=16,
        ):
        counts = self._to_tuple(counts)
        assert isinstance(counts, tuple)
        assert all(isinstance(x, int) for x in counts)
        self._counts = counts
        assert mathtools.is_nonnegative_integer_power_of_two(denominator)
        self._denominator = denominator

    ### SPECIAL METHODS ###

    def __eq__(self, expr):
        r'''Is true when `expr` is a talea with `counts` and `denominator`
        equal to those of this talea. Otherwise false.

        Returns boolean.
        '''
        from abjad.tools import systemtools
        return systemtools.StorageFormatManager.compare(self, expr)

    def __hash__(self):
        r'''Hashes talea.

        Returns integer.
        '''
        from abjad.tools import systemtools
        hash_values = systemtools.StorageFormatManager.get_hash_values(self)
        return hash(hash_values)

    ### PRIVATE PROPERTIES ###

    @property
    def _attribute_manifest(self):
        from abjad.tools import systemtools
        from scoremanager import idetools
        return systemtools.AttributeManifest(
            systemtools.AttributeDetail(
                name='counts',
                command='c',
                editor=idetools.getters.get_nonzero_integers,
                ),
            systemtools.AttributeDetail(
                name='denominator',
                command='d',
                editor=idetools.getters.get_positive_integer_power_of_two,
                ),
            )

    ### PRIVATE METHODS ###

    @staticmethod
    def _to_tuple(expr):
        if isinstance(expr, list):
            expr = tuple(expr)
        return expr

    ### PUBLIC PROPERTIES ###

    @property
    def counts(self):
        r'''Gets counts of talea.

        Returns tuple.
        '''
        return self._counts

    @property
    def denominator(self):
        r'''Gets denominator of talea.

        Returns nonnegative integer power of two.
        '''
        return self._denominator

    ### PUBLIC METHODS ###

    def reverse(self):
        r'''Reverses talea.

        ..  container:: example

            ::

                >>> reversed_talea = talea.reverse()
                >>> print(format(reversed_talea))
                rhythmmakertools.Talea(
                    counts=(1, 1, 4, 2, 3, 1, 2),
                    denominator=16,
                    )

        Returns new talea.
        '''
        counts = tuple(reversed(self.counts))
        result = type(self)(
            counts=counts,
            denominator=self.denominator,
            )
        return result

    def rotate(self, n=0):
        r'''Rotates talea by `n`.

        ..  container:: example

            ::

                >>> rotated_talea = talea.rotate(1)
                >>> print(format(rotated_talea))
                rhythmmakertools.Talea(
                    counts=(1, 2, 1, 3, 2, 4, 1),
                    denominator=16,
                    )

        Returns new talea.
        '''
        counts = tuple(sequencetools.rotate_sequence(self.counts, n))
        result = type(self)(
            counts=counts,
            denominator=self.denominator,
            )
        return result